*! version 1.4.3  22may2007
program _diparm
	version 8.2
	if _caller() < 8.2 {
		_diparm_8 `0'
		exit
	}

	syntax namelist(name=eqs id="equation name(s)") [,	///
		LABel(str)					///
		Function(str)					///
		Derivative(str)					///
		CI(str)						///
		DOF(real -1)					///
		NOProb						///
		Prob						///
		Level(cilevel)					///
		EXP						///
		TANH						///
		ILOGIT INVLOGIT					/// synonyms
		NOTAB						///
	]

	// "NOProb Prob" fudge is OK for programmer's command:
	// 	noprob is the default if a transform is specified
	// 	  prob is the default if no transform specified

	if "`prob'" != "" & "`noprob'" != "" {
		di as err "options prob and noprob cannot be combined"
		exit 198
	}
	else	local prob `prob' `noprob'

	// number of specified equations
	local neq : word count `eqs'

	if "`ilogit'`invlogit'" != "" {
		local ilogit ilogit
	}

	if "`label'" == "" {
		local label = abbrev("/`:word 1 of `eqs''",12) 
	}

	local hardfun `exp' `tanh' `ilogit'
	local nopts = `:word count `hardfun'' + ("`function'" != "")
	if `nopts' > 1 {
		di as err "more than one trasnform specified"
		exit 198
	}

	if   ("`function'"!="" & "`derivative'"=="") ///
	   | ("`function'"=="" & "`derivative'"!="") {
		di in red "both function() and derivative() must be specified"
		exit 198
	}

	// compute the critical value
	tempname z
	if `dof' <= 0 {
		// residual degrees of freedom
		local df_r 0
		Chk4PosInt df_r
	}
	else	local df_r `dof'
	if `df_r' == 0 {
		scalar `z' = invnorm((100 + `level')/200)
	}
	else	scalar `z' = invttail(`df_r',(100-`level')/200)

	// create _tab object:
	// affects -r()- so do it before computing SE, P, and CI

	if "`notab'" == "" {
		tempname Tab
		.`Tab' = ._tab.new, col(7) lmargin(0) ignore(.b)
		// column        1     2     3     4     5     6     7
		.`Tab'.width	13   |12    12     8     8    12    12
		.`Tab'.titlefmt  .     .  %11s   %6s     .  %24s     .
		.`Tab'.pad	 .     2     1     0     2     3     3
		.`Tab'.numfmt    . %9.0g %9.0g %7.2f %5.3f %9.0g %9.0g

		if "`eqs'" == "__sep__" {
			.`Tab'.sep, middle
			exit
		}
		else if "`eqs'" == "__bot__" {
			.`Tab'.sep, bottom
			exit
		}
	}

	if `neq' == 1 {
		// single equation
		tempname b se
		scalar `b' = [`eqs']_cons
		scalar `se' = [`eqs']_se[_cons]
		if `se' == 0 {
			scalar `se' = .
		}

		if "`function'" != "" {
			Function `b' `se' `z' "`function'" "`derivative'"
		}
		else if "`hardfun'" != "" {
			`hardfun' `b' `se' `z'
		}
		else {	// identity transformation
			if ("`prob'" != "noprob") local prob prob
			identity `b' `se' `z'
		}
	}
	else	Multiple "`eqs'" `z' "`function'" "`derivative'" "`ci'"

	AddTest =`df_r', `prob'

	if ("`notab'" != "") exit

	// display row
	if "`prob'" == "prob" {
		.`Tab'.row "`label'" r(est) r(se) r(z) r(p) r(lb) r(ub)
	}
	else {
		.`Tab'.row "`label'" r(est) r(se)   ""   "" r(lb) r(ub)
	}
end

program AddTest, rclass
	syntax =/exp [, Prob ]

	return add
	if "`prob'" == "" {
		return scalar p = .
		return scalar z = .
		exit
	}
	return scalar z = return(est)/return(se)
	if `exp' == 0 {
		return scalar p = 2*norm(-abs(return(z)))
	}
	else	return scalar p = 2*ttail(`exp',abs(return(z)))
end

program Multiple, rclass
	args eqs z func deriv ci
	tempname b V D se
	local neq : word count `eqs'

/* Compute point estimate. */

	MEval "scalar `b'" "`func'" `eqs'

/* Get variance-covariance of desired equations. */

	GetV `eqs'
	mat `V' = r(V)

/* Compute derivatives. */

	mat `D' = J(1,`neq',0)
	local nder : word count `deriv'
	if `neq' != `nder' {
		di in red "number of derivatives not equal to number " /*
		*/ "of parameters"
		exit 198
	}
	tokenize `deriv'
	local eqi 1
	while `eqi' <= `neq' {
		MEval "mat `D'[1,`eqi']" "``eqi''" `eqs'
		local eqi = `eqi' + 1
	}

	mat `V' = `D'*`V'*`D''
	scalar `se' = sqrt(`V'[1,1])
	if `se' == 0 {
		scalar `se' = .
	}

/* Set return. */

	return scalar est = `b'
	return scalar se  = `se'

/* Compute CI. */

	if "`ci'"=="" {
		return scalar lb = `b' - `z'*`se'
		return scalar ub = `b' + `z'*`se'
	}
	else {
		CI`ci' `b' `se' `z'
		return add
	}
end

program CIlogit, rclass
	args p se z
	return scalar lb = 1/(1 + exp(-(log(`p'/(1-`p')) /*
	*/ -`z'*`se'/(`p'*(1-`p')))))
	return scalar ub = 1/(1 + exp(-(log(`p'/(1-`p')) /*
	*/ +`z'*`se'/(`p'*(1-`p')))))
end

program CIprobit, rclass
	args p se z
	tempname y
	scalar `y' = invnorm(`p')
	return scalar lb = normprob(`y'-`z'*`se'/normd(`y'))
	return scalar ub = normprob(`y'+`z'*`se'/normd(`y'))
end

program CIatanh, rclass
	args x se z
	tempname y lb ub ll ul
	scalar `y' = 0.5*log((1+`x')/(1-`x'))
	scalar `lb' = `y'-`z'*`se'/(1-`x'^2)
	scalar `ub' = `y'+`z'*`se'/(1-`x'^2)
	scalar `ll'  = (exp(`lb')-exp(-`lb'))/(exp(`lb')+exp(-`lb'))
	scalar `ul'  = (exp(`ub')-exp(-`ub'))/(exp(`ub')+exp(-`ub'))
	return scalar lb  = cond(`ll'==.,-1,`ll')
	return scalar ub  = cond(`ul'==.,1,`ul')
end

program CIlog, rclass
	args x se z
	return scalar lb = exp(log(`x')-`z'*`se'/`x')
	return scalar ub = exp(log(`x')+`z'*`se'/`x')
end

program GetV, rclass /* get variance-covariance of desired equations */
	tempname V u v
	mat `V' = e(V)
	local eqi 1
	while "``eqi''"!="" {
		mat `u' = `V'[1...,"``eqi'':_cons"]
		mat `v' = nullmat(`v') , `u'
		local eqi = `eqi' + 1
	}
	mat drop `V'
	local eqi 1
	while "``eqi''"!="" {
		mat `u' = `v'["``eqi'':_cons",1...]
		mat `V' = nullmat(`V') \ `u'
		local eqi = `eqi' + 1
	}
	if diag0cnt(`V') > 0 {
		mat `V' = 0*`V'
	}

	return matrix V `V'
end

program MEval /* substitute @1, @2,..., @9 for equation names */
	args genx func
	macro shift 2 /* `*' = list of equation names */

	while "`func'" != "" {
		gettoken sub func : func, parse("@")
		if "`sub'"=="@" {
			gettoken eqi func : func, parse("123456789")
			local exp "`exp'([``eqi'']_cons)"
		}
		else	local exp "`exp'`sub'"
	}

	capture `genx' = `exp'
	if _rc {
		di in red "expression evaluates to" _n "`exp'"
		exit 198
	}
end

program Function, rclass
	args b se z func deriv

	Express `b' `deriv'
	return scalar se = abs(`r(exp)')*`se'

	tempname work
	Express `work' `func' /* macroname `work' substituted into function */

	scalar `work' = `b'
	return scalar est = `r(exp)'

	scalar `work' = `b' - `z'*`se'
	return scalar lb = `r(exp)'

	scalar `work' = `b' + `z'*`se'
	return scalar ub = `r(exp)'

	if return(lb) > return(ub) {
		scalar `work' = return(lb)
		return scalar lb = return(ub)
		return scalar ub = `work'
	}
end

program Express, rclass
	gettoken b 0 : 0

	while "`0'" != "" {
		gettoken sub 0 : 0, parse("@")
		if "`sub'"=="@" {
			local exp "`exp'`b'"
		}
		else	local exp "`exp'`sub'"
	}

	return local exp "`exp'"
end

program exp, rclass
	args b se z
	return scalar est = exp(`b')
	return scalar se  = return(est)*`se'
	return scalar lb  = exp(`b'-`z'*`se')
	return scalar ub  = exp(`b'+`z'*`se')
end

program tanh, rclass
	args b se z
	tempname lb ub ll ul
	scalar `lb' = `b' - `z'*`se'
	scalar `ub' = `b' + `z'*`se'
	return scalar est = (exp(`b')-exp(-`b'))/(exp(`b')+exp(-`b'))
	return scalar se  = (1 - return(est)^2)*`se'
	scalar `ll'  = (exp(`lb')-exp(-`lb'))/(exp(`lb')+exp(-`lb'))
	scalar `ul'  = (exp(`ub')-exp(-`ub'))/(exp(`ub')+exp(-`ub'))
	return scalar lb  = cond(`ll'==.,-1,`ll')
	return scalar ub  = cond(`ul'==., 1,`ul')
end

program ilogit, rclass
	args b se z
	return scalar est = 1/(1 + exp(-`b'))
	return scalar se  = return(est)*(1 - return(est))*`se'
	return scalar lb  = 1/(1 + exp(-(`b'-`z'*`se')))
	return scalar ub  = 1/(1 + exp(-(`b'+`z'*`se')))
end

program identity, rclass
	args b se z
	return scalar est = `b'
	return scalar se  = `se'
	return scalar lb  = `b' - `z'*`se'
	return scalar ub  = `b' + `z'*`se'
end

program Chk4PosInt
	args ename
	if `"`e(`ename')'"' != "" {
		capture confirm integer number `e(`ename')'
		if !c(rc) {
			if `e(`ename')' > 0 {
				c_local `ename' `e(`ename')'
			}
		}
	}
end

exit
