*! version 1.0.1  28feb2006
program _mds_classical, rclass
	version 8

	#del ;
	syntax  anything(name=DM),
	[
		ADDconstant
		DIMensions(numlist integer max=1 >=1)
	] ;
	#del cr

// some checks; beware, I do not check that DM is dissimilarity matrix

	confirm matrix `DM'
	if !issym(`DM') {
		dis as err "symmetric dissimilarity matrix expected"
		exit 503
	}
	if matmissing(`DM') {
		dis as err "classical MDS does not allow missing values"
		exit 198
	}

	local dim `dimensions'

// spectral decomposition of centered distances ///////////////////////////////

	tempname a1 a11 a12 a2 a21 a22 B e E eps F H P Shift Tmp tol V Y

	// one may write a special function to do the double centering
	// b(ij) <-- b(ij) - b(i.) - b(.j) + b(..)

	local n = colsof(`DM')
	matrix `H' = I(`n') - J(`n',`n', 1/`n')
	matrix `B' = `H' * hadamard(`DM',`DM') * `H'
	matrix `B' = -0.25 * (`B'+`B'')

	// we may extract just the largest eigenvectors

	matrix symeigen `V' `E' = `B'

	scalar `tol' = c(epsdouble)^(3/4) * abs(`E'[1,1])
	local np = 0
	forvalues i = 1/`n' {
		if (`E'[1,`i'] > `tol') local ++np
	}

	if `np' == 0 {
		dis as err "no positive eigenvalues found"
		dis as err "classical MDS is not meaningful"
		matlist `E', left(4) row(eigenvalues)
		exit 498
	}

	// smallest constant to make B positive semi-definite
	// (smallest eigenvalue 0, all others strictly positive)

	if ("`addconstant'" != "") & (`E'[1,`n'] < 0) {
		scalar `Shift' = -2*`E'[1,`n']
		matrix `B' = `B' + `Shift'*`H'
		matrix symeigen `V' `E' = `B'

		scalar `tol' = c(epsdouble)^(3/4) * abs(`E'[1,1])
		local np = 0
		forvalues i = 1/`n' {
			if (`E'[1,`i'] > `tol') local ++np
		}
	}
	else {
		scalar `Shift' = 0
	}
	matrix drop `B' `H'

// determine number of relevant eigenvalues ///////////////////////////////////

	if "`dim'" != "" {
		if `dim' > `np' {
			dis as err "dim() exceeds number of positive "  ///
			    "eigenvalues of the centered dissimilarty " ///
			    "matrix (`np')"
			exit 496
		}
		local p = `dim'
	}
	else {
		local p = min(`np',2)
		// dis as txt "(defaults to `p' dimensions; max = `np')"
	}

// approximating configuration Y /////////////////////////////////////////////

	matrix `V' = `V'[1..., 1..`p']
	matrix rownames `V' = `:rownames `DM''

	matrix `e' = `E'[1, 1..`p']
	forvalues i = 1 / `p' {
		local clist `clist' dim`i'
		matrix `e'[1,`i'] = sqrt(`e'[1,`i'])
	}
	matrix colnames `E' = `:rownames `DM''

	matrix `Y' = `V' * diag(`e')
	matrix colnames `Y' = `clist'
	matrix rownames `Y' = `:rownames `DM''

// is Y uniquely determined //////////////////////////////////////////////////

	local unique = 1
	forvalues i = 1 / `=min(`p',`n'-1)' {
		if reldif(`E'[1,`i'],`E'[1,`i'+1]) ///
		   < c(epsdouble)^(3/4) * abs(`E'[1,1])	{
			local unique = 0
		}
	}

// Determine OLS-linear transformation:  Fitted = transform(DM) + eps ///////

	tempname b

	matrix `b'= (.)
	mata: Linear("`DM'", "`Y'", "`b'")

	matrix colnames `b' = _cons dissim
	matrix rownames `b' = distance

// Mardia's fit measures //////////////////////////////////////////////////////

	scalar `a11' = 0
	scalar `a21' = 0
	forvalues i = 1 / `p' {
		scalar `a11' = `a11' + abs(`E'[1,`i'])
		scalar `a21' = `a21' + `E'[1,`i']^2
	}

	scalar `a12' = `a11'
	scalar `a22' = `a21'
	forvalues i = `=`p'+1' / `n' {
		scalar `a12' = `a12' + abs(`E'[1,`i'])
		scalar `a22' = `a22' + `E'[1,`i']^2
	}

	scalar `a1' = `a11' / `a12'
	scalar `a2' = `a21' / `a22'

// ereturn ////////////////////////////////////////////////////////////////////

	return local  method  classical
	return local  unique  `unique'

	return scalar N       = `n'
	return scalar p       = `p'
	return scalar np      = `np'
	return scalar addcons = `Shift'
	return scalar mardia1 = `a1'
	return scalar mardia2 = `a2'

	return matrix linearf = `b'    // Distance = a + b*Dissimilarity
	return matrix Ev      = `E'    // eigenvalues
	return matrix Y       = `Y'    // approx config
end


// returns in Bname the Least-squares linear transform T
//
//    eu-distance_ij} = T.a + T.b*dissim_{ij}   for i<>j
//
// Note that we exclude the diagonal
//
mata:
void function Linear( string scalar Dname, ///
			   string scalar Yname, ///
			   string scalar Tname )
{
	real scalar  b, dY, F, i, j, m, mD, mD2, mF, mDF, n
	real matrix  B, D, Y

	D = st_matrix(Dname)
	Y = st_matrix(Yname)

	// we do not assume that D is symmetric	and allow for missings

	mD  = 0
	mD2 = 0
	mF  = 0
	mDF = 0
	m   = 0

	n   = cols(D)
	for (i=1; i<=n; i++) {
		for (j=1; j<=n; j++) {
			if ((i ~= j) & (D[i,j] < .)) {
				dY = Y[i,.] - Y[j,.]
				F  = sqrt(dY*dY')

				if (F < .) {
					mD  = mD  + D[i,j]
					mD2 = mD2 + D[i,j]*D[i,j]
					mF  = mF  + F
					mDF = mDF + D[i,j]*F
					m   = m + 1
				}
			}
		}
	}

	mD  = mD  / m
	mD2 = mD2 / m
	mF  = mF  / m
	mDF = mDF / m

	b = (mDF - mF*mD)/(mD2 - mD*mD)
	B = (mF-b*mD, b)

	st_matrix(Tname, B)
}
end
