*! version 1.1.3  02jan2007
program _robust2, rclass sort
	version 9
	syntax varlist(numeric) [if] [in]	///
		[pw iw] [,			///
		SVY				///
		CLuster(varname)		///
		SUBpop(passthru)		///
		Variance(name)			///
		VSRS(name)			///
		minus(numlist max=1 int >=0)	///
		ZEROweight			///
		TOUSE1(name)			///
	]

	if "`minus'" == "" {
		local minus 1
	}
	if "`svy'" != "" & "`weight'" != "" {
		di as err ///
"weights can only be supplied to {help svyset##|_new:svyset}"
		exit 198
	}
	if "`svy'" != "" | "`weight'" == "iweight" {
		local zeroweight zeroweight
	}
	marksample touse, `zeroweight'

	if "`variance'" != "" {
		confirm matrix `variance'
	}
	else {
		if "`e(V)'" != "matrix" {
			error 301
		}
		local vartmp yes
		tempname variance
		matrix `variance' = e(V)
	}
	local k_scores : word count `varlist'
	tempname eqdims
	// verify # vars in varlist == # equations is `variance'
	CheckMatrixLabels `k_scores' `variance' `eqdims'
	local xvarlist	`"`r(varlist)'"'
	local eqnames	`"`r(eqnames)'"'
	local xlist : subinstr local xvarlist "_cons" "`touse'", all word
	local marko : subinstr local xvarlist "_cons" "", all word
	markout `touse' `marko'

	// get the -svyset- information
	tempvar subuse
	_svy_setup `touse' `subuse', `svy' `subpop'
	return add

	local nobs1 = c(N) - return(N) + 1
	if `nobs1' != 1 & "`posts'" != "" {
		// sort the data by the estimation sample, then use -in-
		sort `touse'
		local ifin "in `nobs1'/l"
		local i1i2 "(`nobs1', `c(N)')"
	}
	else {
		if `nobs1' != 1 {
			// no need to sort, so just use -if-
			local ifin if `touse'
		}
		local i1i2 "."
	}

	return local settings
	return local brrweight
	return local jkrweight
	if "`svy'" != "" {
		local stages	= return(stages)
		if missing(`stages') {
			local stages 1
		}
		forval i = 1/`stages' {
			if "`return(strata`i')'" != "" {
				local stri `return(strata`i')'
				if substr("`:type `stri''",1,3) == "str" {
					tempvar stru
					qui egen `stru' = group(`stri') `ifin'
					local stri `stru'
				}
				local strata `strata' `stri'
				local sortlist `sortlist' `stri'
			}
			else {
				// NOTE: `sui' exists because you
				// cannot have stratification at level
				// i if the sampling units are the
				// observations at level i-1
				if `i' == 1 {
					if `stages' > 1 {
						local strata	`touse'
					}
				}
				else	local strata	`strata' `sui'
			}
			if "`return(su`i')'" != "" {
				local sui `return(su`i')'
				if substr("`:type `sui''",1,3) == "str" {
					tempvar sutmp
					qui egen `sutmp' = group(`sui') `ifin'
					local sui `sutmp'
				}
				local su `su' `sui'
				local sortlist `sortlist' `sui'
			}
			// else there is one less su var than stages
			local fpc	`fpc' `return(fpc`i')'
		}
		local posts	`return(poststrata)'
		local postw	`return(postweight)'
		local wtype	`return(wtype)'
		local wexp	`"`return(wexp)'"'
		if "`wexp'" != "" {
			tempvar wvar
			quietly gen double `wvar' `wexp'
		}
	}
	else {
		if "`cluster'" != "" {
			if substr("`:type `cluster''",1,3) == "str" {
				tempvar clvar
				qui egen `clvar' = group(`cluster') `ifin'
			}
			else	local clvar `cluster'
		}
		if "`weight'" != "" {
			tempvar wvar
			quietly gen double `wvar' `exp' `ifin'
			local wexp `"`:list retok exp'"'
		}
		local stages	1
		local su	`clvar'
		local sortlist	`su'
		local wtype	`weight'
	}
	if "`vartmp'" == "yes" {
		return matrix V `variance'
	}

	// poststratification
	if "`posts'" != "" {
		tempvar postid
		sort `touse' `posts'
		GenId `postid' `posts' `nobs1'
		local postvars `postid' `postw'

		// do not return vsrs() for poststratification
		local vsrs
	}

	local mvarlist	`varlist'	///
			`xlist'		///
			`subuse'	///
			`wvar'		///
			`fpc'		///
			`strata'	///
			`su'		///
			`postvars'
	local mvarlist	: list retok mvarlist
	local kscores	: list sizeof varlist
	local kxlist	: list sizeof xlist
	local ksubuse	: list sizeof subuse
	local kwvar	: list sizeof wvar
	local kfpc	: list sizeof fpc
	local kstrata	: list sizeof strata
	local ksu	: list sizeof su
	local kpostvars	: list sizeof postvars

	if `"`sortlist'"' != "" {
		sort `touse' `sortlist'
	}
	tempname v
	matrix `v' = `variance'
	capture			///
	mata: _robust2(		///
		"`v'",		/// OUTPUT: variance estimates
		"`vsrs'",	/// OUTPUT: SRS variance estimates
		`i1i2',		/// INPUT:  (i1, i2)
		"mvarlist",	/// INPUT:  local holding stata variables
		"`touse'",	/// INPUT:  touse variable
		`kscores',	/// INPUT:  # of score vars
		`kxlist',	/// INPUT:  # of x vars
		"`eqdims'",	/// INPUT:  stata matrix of equation dims
		`ksubuse',	/// INPUT:  .,0,1 for subuse var
		`kwvar',	/// INPUT:  .,0,1 for for wvar
		`stages',	/// INPUT:  # of stages
		`kfpc',		/// INPUT:  # of fpc vars
		`kstrata',	/// INPUT:  # of strata vars
		`ksu',		/// INPUT:  # of sample unit vars
		`kpostvars',	/// INPUT:  .,0,2 for poststrata vars
		`minus'		/// INPUT:  from minus() option
	)

	if c(rc) error c(rc)

	// saved results
	matrix `variance'[1,1] = `v'
	if "`vsrs'" != "" {
		local Vlist `vsrs' `vsrs'sub
		if "`vsrs'" != "" & "`fpc'" != "" {
			local Vlist `Vlist' `vsrs'wr `vsrs'subwr
		}
	}
	return scalar census = r(census)
	foreach mat of local Vlist {
		matrix colna `mat' = `xvarlist'
		matrix rowna `mat' = `xvarlist'
		matrix coleq `mat' = `eqnames'
		matrix roweq `mat' = `eqnames'
	}
	local sclist : r(scalars)
	foreach sc of local sclist {
		return scalar `sc' = r(`sc')
	}
	local matlist : r(matrices)
	tempname m
	foreach mat of local matlist {
		matrix `m' = r(`mat')
		return matrix `mat' `m'
	}
	if "`svy'" != "" {
		return scalar df_r = r(N_clust) - r(N_strata)
		if `"`subpop'"' == "" {
			return local N_sub
			return local N_subpop
		}
	}
	else {
		return local N_sub
		return local N_subpop
		if inlist("`wtype'", "fweight", "iweight") {
			return scalar N = r(N_pop)
			return scalar df_r = r(N_pop) - 1
		}
		else	return scalar df_r = r(N) - 1
		if "`wtype'" != "" {
			return local wtype `wtype'
			return local wexp  `"`wexp'"'
		}
		return local N_pop
		return local N_strata
		return local N_psu
	}
	if "`touse1'" != "" {
		capture confirm new var `touse1'
		if c(rc) {
			quietly replace `touse1' = `touse'
		}
		else	rename `touse' `touse1'
	}
end

program CheckMatrixLabels, rclass
	args k v eqdims
	local cons _cons
	local eqnames	: coleq `v', quote
	local ueq	: list uniq eqnames
	local names	: colna `v'

	local k_eq : word count `ueq'
	if `k' < `k_eq' {
		error 102
	}
	else if `k' > `k_eq' {
		error 103
	}
	local vlist : subinstr local names "_cons" "", all word
	if "`:list uniq vlist'" != "" {
		confirm var `vlist'
	}

	matrix `eqdims' = J(1,`k',0)
	forval i = 1/`k' {
		local eq : word `i' of `ueq'
		local ignored : subinstr local eqnames `""`eq'""' "", ///
			all word count(local cnt)
		if `i' == 1 {
			matrix `eqdims'[1,`i'] = `cnt'
		}
		else {
			matrix `eqdims'[1,`i'] = `eqdims'[1,`i'-1]+`cnt'
		}
	}
	return local varlist	`"`names'"'
	_mat_clean_coleq eqnames : `eqnames'
	return local eqnames	`"`eqnames'"'
end

program GenId
	// Assumption:  the data is sorted by a "touse" variable and the
	// "oldid" variable.
	//
	// Note: this routine is much faster than using -egen, group()- or
	// -by: generate- with an -if-, given the above assumption.
	args newid oldid nobs1
	local in `nobs1'/l
	quietly gen `newid' = `oldid' != `oldid'[_n-1] in `in'
	quietly replace `newid' = 1 in `nobs1'
	quietly replace `newid' = sum(`newid') in `in'
end

exit
