*! version 2.0.3  18aug2006
* asmprobit - multinomial probit model for alternative specific
* 		and case specific variables
	
program asmprobit, eclass byable(onecall) sortpreserve
	version 9

	if _by() {
		local BY `"by `_byvars'`_byrc0':"'
	}
	if replay() {
		if `"`BY'"' != "" error 190
		if `"`e(cmd)'"' != "asmprobit" error 301
		Replay `0'
		exit
	}
	qui syntax [anything] [fw pw iw] [if] [in], case(varname) [ ///
		Cluster(varname) VCE(passthru) *]


	if `"`vce'"' != "" {
		tempname id
		local idopt idcluster(`id')
		local case0 "`case'"
		if `"`cluster'"' == "" {
			quietly gen `id' = `case'
			local clopt cluster(`case')
			local caopt case(`id')
		}
		else {
			quietly gen `id' = `cluster'
			local clopt cluster(`cluster')
			local caopt case(`case')
			local bsgropt group(`case')
		}
		if "`weight'" != "" {
			local wgt [`weight'`exp']
		}
		local vceopts jkopts(`clopt') ///
			bootopts(`clopt' `idopt' `bsgropt') ///
			required(BASEalternative SCALEalternative) ///
			mark(case ALTernatives CASEVars CLuster)
		`by' _vce_parserun asmprobit, `vceopts'	: ///
			`anything' `wgt' `if' `in', ///
			`caopt' `vce' `options'
		if "`s(exit)'" != "" {
			ereturn local casevar `case'
			ereturn local clustvar `cluster'
			if "`cluster'" == "" {
				local cmd1 `"`e(command)'"'
				local cmd2 : subinstr local cmd1 "`id'" "`case'"
				ereturn local command `"`cmd2'"'
			}
			exit
		}
		qui syntax [anything] [fw pw iw] [if] [in], case(varname) [ * ]
		local 0 `"`anything' [`weight'`exp'] `if' `in', case(`case0') `options'"'
	}
	cap noi `BY' Estimate `0'
	local rc = _rc
	cap mata : mata drop _mprobit_panel_info
	macro drop MPROBIT_*
	exit `rc'
end

program Estimate, eclass byable(recall)
	syntax varlist(numeric) [if] [in] [fw pw iw],	/// 
		case(varname numeric)			///
		ALTernatives(varname) [			///
		noCONstant				///
		CASEVars(varlist numeric)		///
		BASEalternative(string)			///
		SCALEalternative(string)		///
		noTRANsform				///
		CORRelation(string)			///
		STDdev(string)				///
		STRUCtural				///
		INTMethod(string)			///
		INTPoints(integer 0)			///
		INTBurn(integer 0)			///
		INTSeed(string) 			///
		ANTIthetics				///
		CONSTraints(numlist > 0 integer)	///
		Robust					///
		CLuster(varname)			///
		FROM(string)				///
		TECHnique(string)			///
		INITBhhh(integer 0)			///
		ITERate(passthru)			///
		VCE(string)				///
		NOPREserve				///
		Level(cilevel)				///
		DEBug					///
		MLInit					///
		TRace					///
		noLOg					///
		noDROP					///
		* ]

	gettoken choice attribs : varlist

	mlopts mlopts rest, `options'
	if "`s(collinear)'" != "" {
		local drop nodrop
	}
	PostError , `rest'
	local diopts level(`level')

	local nattrib: word count `attribs'
	local ncasev: word count `casevars'
	if `nattrib'==0 & `ncasev'==0 & "`constant'"!="" {
		di as error "too few variables specified"
		exit 102
	} 

	marksample touse
	markout `touse' `case' `casevars'
	markout `touse' `alternatives', strok
	if (`"`cluster'"'!="") {
		markout `touse' `cluster', strok
		local robust robust
	}
	qui count if `touse'
	if r(N) == 0 {
		di as error "no cases remain after excluding invalid observations"
		exit 2000
	}

	cap assert float(`case')==float(floor(`case')) if `touse'
	if _rc != 0 {
		di as error "case() variable `case' must be coded as integers"
		exit 459 
	}
	cap assert `choice'==0 | `choice'==1 if `touse' 
	if _rc != 0 {
		di as error "{p}depvar `choice' must be a 0-1 variable indicating " ///
		 "which alternative is the outcome{p_end}"
		exit 450
	}

	tempvar altern altlabels
	cap egen `altern' = group(`alternatives') if `touse', lname(`altlabels')
	if _rc > 0 {
		/* should not happen */
		di as error "failed to renumber alternative levels"
		error _rc
	}
	sort `case' `altern'
	tempname casestat
	local bfwts = 0
	if "`weight'" != "" {
		tempvar wt 
		qui gen `wt' `exp'
		local bfwts = ("`weight'"=="fweight")
	}
	mata: _mprobit_validate_choice(`"`choice'"',`"`altern'"',`"`case'"', ///
		`"`casevars'"', `"`wt'"', `bfwts', `"`touse'"', `"`casestat'"')
	if `casestat'[5,1] == 0 {
		di as error "no cases remain after excluding invalid observations"
		exit 2000
	}
	if `casestat'[5,1] == 1 {
		di as error "only one case remains after excluding invalid observations"
		exit 2001
	}
	if `casestat'[6,1] > 0 {
		drop `altern'
		cap egen `altern' = group(`alternatives') if `touse', lname(`altlabels')
		if _rc > 0 {
			/* should not happen */
			di as error "failed to renumber alternative levels"
			error _rc
		}
		sort `case' `altern'
	}
	local balanced = `casestat'[4,1]
	preserve
	qui keep if `touse'
	if "`drop'" == "" {
		/* remove collinear variables */
		_rmcoll `attribs', noconstant
		local attribs `"`r(varlist)'"'
		local nattrib: word count `attribs'

		_rmcoll `casevars', `constant'
		local casevars `"`r(varlist)'"'
	}

	macro drop MPROBIT_*

	local fullcov = 1
	local gencorstr = 1
	local cholesky = 1
	local scale = 1
	local lstd = length("`stddev'")
	if "`stddev'"=="" | "`stddev'"==substr("heteroskedastic",1,max(3,`lstd')) {
		local stddev "heter"
	}
	else if "`stddev'" == "probit" {
		/* undocumented */
		local scale .5
		local stddev "heter"
	}
	else if "`stddev'" == substr("homoskedastic",1,max(3,`lstd')) {
		local stddev "homo"
		local cholesky = 0
		local fullcov = 0
	}
	else {
		MatrixName `stddev', type(stddev)
		local stddev `r(name)'
		local varmattype `r(matspec)'
		local cholesky = 0
		local fullcov = 0
	}
	local varmat = ("`varmattype'"!="")
	local structcov = `varmat' | ("`structural'"!="")
	local lcor = length("`correlation'")
	if "`correlation'"=="" | "`correlation'"==substr("unstructured",1,max(3,`lcor')) {
		local correlation "unstruct"
		local cholesky = ("`stddev'"=="heter")
		if "`from'"!="" & `cholesky' {
			if (`structcov' & "`transform'"=="")  local cholesky = 0
		}
		else if (!`structcov') local structcov = (!`cholesky') 
	}
	else if "`correlation'" == substr("independent",1,max(3,`lcor')) {
		local cholesky = 0
		local fullcov = 0
		local correlation "indep"
		local structcov = 1
	}
	else if "`correlation'" == substr("exchangeable",1,max(3,`lcor')) {
		local cholesky = 0
		local fullcov = 0
		local correlation "exch" 
		local structcov = 1
	}
	else {
		MatrixName `correlation', type(correlation)
		local correlation `r(name)'
		local cormattype `r(matspec)'

		local cholesky = 0
		local fullcov = 0
		local gencorstr = 0
		local structcov = 1
		tempname corstr
		matrix `corstr' = `correlation'
		/* save the name of the original matrix */
		local cormat = "`correlation'"
		local correlation user
	}
	if `structcov' & `scale'==.5 {
		di as error "stddev(probit) cannot be used with the structural " ///
		 "covariance parameterization"
		exit 198
	}
	local antithetics = ("`antithetics'"!="") 
	if ("`intmethod'"=="") local imeth = 2
	else {
		local lmth = length("`intmethod'")

		if ("`intmethod'"==substr("halton",1,max(3,`lmth'))) local imeth = 1
		else if ("`intmethod'"==substr("hammersley",1,max(3,`lmth'))) local imeth = 2
		else if ("`intmethod'"==substr("random",1,max(3,`lmth'))) local imeth = 3
		else {
			di as error "option intmethod() must specify halton, hammersley, " ///
			 "or random" 
			exit 198
		}
	}		

	local iff if `choice'==1
	if ("`cluster'"!="") local clopt2 cluster(`cluster')
       	if ("`robust'"!="" | "`weight'"=="pweight") local pseudo pseudo
	if "`weight'"=="iweight" | "`weight'"=="fweight" {
		local wtoptinit [`weight'`exp']
		local wtopt [`weight'`exp']
	}
	else if "`weight'" == "pweight" {
		local wtopt [iweight`exp']	
	}
	
	tempname corfix
	/* do not use frequencies */
	cap confirm string variable `alternatives'
	if _rc == 0 {
		qui tabulate `altern'
	}
	else {
		tempname altlevels
		qui tabulate `alternatives', matrow(`altlevels')
		local altlabels : value label `alternatives'
	}

	local nalt = r(r)
	if `nalt' > 20 {
		di as error "there are `nalt' alternatives in `alternatives'; " ///
		 "the maximum number of alternatives is 20"
		exit 149
	}
	
	AlternativeIndex "`altlevels'" `nalt' "`altlabels'" "`basealternative'" ///
		"basealternative" 
	local ibase = r(index)
	AlternativeIndex "`altlevels'" `nalt' "`altlabels'" "`scalealternative'" ///
		"scalealternative"
	local iscale = r(index)
	if `iscale'<. & `iscale'==`ibase' {
		di as error "scalealternative() cannot equal the basealternative()"
		exit 459	
	}

	if `gencorstr' {
		CorStruct `nalt' "`correlation'" "`stddev'" `ibase' `iscale' `scale' ///
			`structcov' `varmat' "`varmattype'"
		tempname corstr
	}
	else {
		ValidateCorStruct "`corstr'" `nalt' "`cormat'" "`stddev'" `ibase' `iscale' ///
			"`cormattype'" `varmat' "`varmattype'"
	}
	tempname order0
	local ncor = r(ncor)
	local nvar = r(nvar)
	local ibase = `r(ibase)'
	local iscale = `r(ifixed)'
	matrix `corstr' = r(corstr)
	matrix `corfix' = r(corfix)
	matrix `order0' = r(order)

	local ncov = `ncor' + `nvar'
	if (!`structcov') local transform "transform"
	else if (`ncov'==0) local cholesky = 0
	local reparam = (`cholesky' & "`transform'"=="")

	local d1 d1
	ParseTech `"`technique'"' `initbhhh' "`vce'" 

	local technique `"`s(technique)'"'
	local tech `"`s(tech)'"'
	local d1 "`s(d1)'"
	local vce0 "`s(vce0)'"
	local vce "`s(vce)'"
	local tech2 "`s(tech2)'"
	local vce2 "`s(vce2)'"
	local d21 "`s(d21)'"
	local initbhhh = `s(initbhhh)'
	local techmeth "`s(techmeth)'"
	local techswitch = `s(techswitch)'
		
	if `nalt' <= 2 {
		local intpoints = 0
		local intburn = 0
		local initbhhh = 0 
	}
	else {
		if `intpoints' < 0 {
			di as error "intpoints() must be greater than 0"
			exit 198
		}
		if `intpoints' == 0 {
			/* number of points in the sequence */
			if (`imeth'==3) local intpoints  = 100*`nalt'
			else local intpoints = 50*`nalt'
		}
		if `intburn' < 0 {
			di as error "intburn() must be greater than 0"
			exit 198
		}
		else if `intburn'>0 & `imeth'==3 {
			di as error "intburn() cannot be used with intmethod(random)"
			exit 198
		}
		local critopt crittype(log simulated-`pseudo'likelihood)
	}

	global MPROBIT_NALT = `nalt'
	global MPROBIT_ALTERNATIVES "`altern'"
	global MPROBIT_ID "`case'"
	if ("`intseed'" == "") global MPROBIT_SEED = "`c(seed)'"
	else if (`imeth' == 3) 	global MPROBIT_SEED = "`intseed'"
	else {
		di as error "{p}intseed() is not allowed when specifying the " ///
		 "Halton or Hammersley sequences{p_end}"
		exit 184
	}
	
	global MPROBIT_METHOD = `imeth'
	global MPROBIT_BURN = `intburn'
	global MPROBIT_TRACE = ("`trace'"!="")
	if (`reparam') {
		global MPROBIT_CHOLESKY 1
		local cholesky 0
	}
	else global MPROBIT_CHOLESKY = `cholesky'
	global MPROBIT_FULLCOV = `fullcov'
	global MPROBIT_CORRELATION "`correlation'"
	global MPROBIT_ANTITHETICS = `antithetics'
	global MPROBIT_CORSTRUCT "`corstr'"
	global MPROBIT_CORFIXED "`corfix'"
	global MPROBIT_STRUCTCOV = `structcov'
	global MPROBIT_ORDER "`order0'"
	global MPROBIT_NVAR = `nvar'
	global MPROBIT_NCOR = `ncor'
	global MPROBIT_ATTRIBS `"`attribs'"'
	global MPROBIT_NATTRIB = `nattrib'
	global MPROBIT_CASEVARS  `"`casevars'"'
	global MPROBIT_BALANCED = `balanced'
	global MPROBIT_BASE = `ibase'
	global MPROBIT_WTOPT `"`wtopt'"'
	
	if (`nalt' == 2) local intpoints = 0
	global MPROBIT_NPOINTS = `intpoints'	

	if `ncov' > 0 {
		forvalues i=1/`ncov' {
			tempvar v`i'
			gen double `v`i'' = 0.0
			local vscrs `"`vscrs' `v`i''"'
		}
		global MPROBIT_VSCRS `vscrs'
	}
	if `nattrib' > 0 {	
		forvalues i=1/`nattrib' {
			tempvar a`i'
			gen double `a`i'' = 0.0
			local ascrs `"`ascrs' `a`i''"'
		}
		global MPROBIT_ASCRS `ascrs'
		local wald = 1
	}
	else local wald = 0

	GetModel `choice', case(`case') attribs(`attribs') ibase(`ibase') ///
		alternatives(`altern') altname(`alternatives') `constant' ///
		casevars(`casevars') 

	local ncoef = `r(ncoef)'
	if `casestat'[5,1] <= 10*(`ncov'+`ncoef') {
		if `casestat'[5,1] <= `ncov'+`ncoef' {
			di as error "there are only `=`casestat'[5,1]' cases " ///
			 "to estimate `=`ncov'+`ncoef'' parameters"
			exit 2001
		}
		di in gr "{p 0 7}note: there are only `=`casestat'[5,1]' cases " ///
		 "to estimate `=`ncov'+`ncoef'' parameters{p_end}"
	}
	local altmodel `"`r(altmodel)'"'
	local casemodels `"`r(casemodels)'"'
	local rcfnms `"`r(rcfnms)'"'

	global MPROBIT_NCASECOEF = `r(ncasecf)'
	global MPROBIT_CONST = `r(const)'
	global MPROBIT_ALTEQS `"`r(alteqs)'"'
	if $MPROBIT_NCASECOEF > 0 {
		local wald = `wald' + `nalt' - 1
	}
	tempname b order ic ilog
	if "`from'" == "" {
		local vconstr = 0
		if `"`constraints'"' != "" {
			VParStripe 0
			matrix `b' = J(1,`ncoef'+`ncov',0)
			matrix colnames `b' = `rcfnms' `r(vcfnms)'

			CheckConstraints "`b'" `ncov' `"`constraints'"'
			/* vconstr is the number of constraints on the variance parameters 	*/
			/* bconstr is the number of constraints on the regression parameters 	*/
			local vconstr = `e(vconstr)'
			local bconstr = `e(bconstr)'
			if `bconstr' > 0 {
				/* pass the constraint matrix on to clogit for inital estimates */
				tempname C 
				matrix `C' = e(C)
				local cmat cmat(`C')
			}	
		}
		if `reparam' & `vconstr' > 0 {
			/* constraints on variance/covariance parameters, cannot */
			/* reparameterize 					 */
			local reparam = 0
			global MPROBIT_CHOLESKY = `cholesky'
		}
	}
	if $MPROBIT_CHOLESKY {
		/* pivot base and scale to 1st and second position */
		matrix `order' = J(`nalt',1,2)
		matrix `order'[1,1] = `ibase'
		if `iscale' < . {
			matrix `order'[2,1] = `iscale'
			local j = 3
		}
		else local j = 2

		local k = 1
		local reorder = (`order'[1,1]!=1 | `order'[2,1]!=2)
		forvalues i=`j'/`nalt' {
			while `k'==`ibase' | `k'==`iscale' {
				local `++k'
			}
			matrix `order'[`i',1] = `k++'
			local reorder = (`reorder' | `order'[`i',1]!=`i')
		}
		if (`reorder')	{
			PivotMat "`corstr'" "`order'" 1
			PivotMat "`corfix'" "`order'" 0
		}
		global MPROBIT_ORDER = "`order'"
	}
	if "`from'"=="" {
		if "`mlinit'"=="" {
			/* cannot use ml search(on) since it will try to scale the coefficients */
			InitMProbit `choice' `iff' `wtoptinit', case(`case') attribs(`attribs') /// 
				alternatives(`altern')  ibase(`ibase') casevars(`casevars')     ///
				`constant' `cmat' coefnames(`rcfnms')

			matrix `b' = r(from)
			local seaopt search(off)
			local initopt init(`b')
		}
		else local seaopt search(norescale) 
	}
	else {
		local seaopt search(off)
		local initopt init(`from')
	}
	if !`structcov' {
		matrix `corfix'[2,2] = sqrt(`corfix'[1,1]+`corfix'[2,2])
		matrix `corstr' = `corstr'[2..`nalt',2..`nalt']
		matrix `corfix' = `corfix'[2..`nalt',2..`nalt']
		if ("`from'"=="" & "`mlinit'"=="") matrix `b' = sqrt(`scale')*`b'
	}
	if `ncov' > 0 {
		VParStripe $MPROBIT_CHOLESKY
		local thetas `"`r(thetas)'"'
		local vcfnms `"`r(vcfnms)'"'
		if "`from'"=="" & "`mlinit'"=="" {
			if !`structcov' {
				InitDifferencedCov `nalt' `scale'
				matrix `b' = (`b',r(b))
			}
			else {
				matrix  `b' = (`b',J(1,`ncov',0))
			}
			matrix colnames `b' = `rcfnms' `vcfnms'
		}

		GenTmat `nvar' `ncor' "`corstr'" 
		tempname T
		matrix `T' = r(T)
		global MPROBIT_TMAT `T'
	}
	forvalues i=1/$MPROBIT_NCASECOEF {
		tempvar c`i'
		gen double `c`i'' = 0.0
		local cscrs `"`cscrs' `c`i''"'
	}
	global MPROBIT_CSCRS `"`cscrs'"'
	if $MPROBIT_NCASECOEF>0 & "`pseudo'"!="" {
		/* check that we will be able to generate scores */
		/*  after estimation				 */
		CheckMemoryForRobust
	}

	if "`tech2'"=="" & `reparam' {
		local tech2 nr
		local vce2  oim
		if ("`vce'"=="opg") local d21 d2
		else local d21 d1
		if (!`techswitch') local vceopt vce(native)
	}
	if "`tech2'" != "" {
		local waldopt2 waldtest(`wald')
		local waldopt waldtest(0)
		if (!`techswitch') local vceopt vce(native)
	}
	else {
		if ("`vce'"!="") local vceopt vce(`vce')
		local waldopt waldtest(`wald')
	}
	if ("`constraints'" != "") local conopt constraints(`constraints')

	local converged = 0
	local rc = 0
	if `initbhhh' > 0 {
		if "`log'" == "" {
			di _n in smcl in gr "(setting optimization to BHHH)"
			if ("`techmeth'" != "") local log1 log
		}
		/* `onercp'!=0, we use d2 and nr but the algorithm is actually 	*/
		/*  bhhh technique and opg for negative hessian 		*/
		ml model d2 asmprobit_lf `altmodel' `casemodels' `thetas' `iff'  ///
			`wtopt', technique(nr) collinear nopreserve `conopt' 	 ///
			noscvars `critopt' `seaopt' `initopt' nooutput `trace'   ///
			iterate(`initbhhh') `log' nowarning max 

		matrix `b' = e(b)
		scalar `ic' = e(ic)
		matrix `ilog' = e(ilog)
		local initopt init(`b')
		local seaopt search(off)
		local converged = e(converged)
		local rc = e(rc)
		if `converged' & "`tech2'"=="" {
			local tech2 nr
			local vce2  oim
			if ("`vce'"=="opg") local d21 d2
			else local d21 d1
			local waldopt2 waldtest(`wald')
		}
	}
	if `converged' == 0 {
		if "`debug'" != "" {
			global MPROBIT_TRACE = 0

			ml model d1debug asmprobit_lf `altmodel' `casemodels' `thetas' `iff' ///
				`wtopt', nopreserve collinear `initopt' noscvars gradient    ///
				showstep trace max search(off) `trace' `log' 
		}
		else {
			if ("`log1'" != "") di _n in smcl in gr "(switching optimization to " ///
				"`techmeth')"
			/* let -ml- approximate the negative hessian (unless tech=bhhh) */
			ml model `d1' asmprobit_lf `altmodel' `casemodels' `thetas' `iff' ///
				`wtopt', technique(`tech') nopreserve collinear `conopt'  ///
				`initopt' `seaopt' noscvars `vceopt' `robopt' `iterate'   ///
				`mlopts' `critopt' `waldopt' nooutput max `trace' `log'
		}

		matrix `b' = e(b)
		scalar `ic' = e(ic)
		matrix `ilog' = e(ilog)
		local converged = e(converged)
		local rc = e(rc)
	}
	if "`tech2'" != "" {
		if `reparam' {
			/* put parameter estimates into desired metric */
			if "`log'" == "" {
				di _n in gr "Reparameterizing to correlation metric" _c
				if (`converged') di " and refining estimates" 
				else di _n
			}
			Reparameterize "`b'" `"`rcfnms'"' "`order0'"

			local thetas `"`r(thetas)'"'
			global MPROBIT_CHOLESKY = 0
			if `converged' {
				local iteropt `iterate'
			}
			else {
				local iteropt iterate(0)
				local qui qui	
			}
		}
		else {
			local iteropt iterate(0)
			local qui qui 
		}
		`qui' ml model `d21' asmprobit_lf `altmodel' `casemodels' `thetas' `wtopt' ///
			`iff', technique(`tech2') vce(`vce2') collinear `conopt' init(`b') ///
			nopreserve `mlopts' `critopt' `waldopt2' search(off) nooutput      ///
			noscvars max nowarning `iteropt' `log'
		if `reparam' & e(converged)==0 {
			di in gr "note: reparameterization failed"
		} 
		else {
			ereturn scalar converged = `converged'
			ereturn scalar rc = `rc'
		}
	}
	if "`pseudo'" != "" {
		tempname V b g negH lnf
		global ML_y1 = "`choice'"
		scalar `lnf' = 0
		matrix `V' = e(V)
		matrix `b' = e(b)

		asmprobit_lf 3 "`b'" "`lnf'" "`g'" "`negH'"  
		/* temporary varaibles for covariance scores */
		if `nattrib' > 0 {
			/* dummy names for coefficients */
			foreach  scr of global MPROBIT_ASCRS {
				local robnms `"`robnms' `scr':_cons"'
			}
		}
		if $MPROBIT_NCASECOEF > 0 {
			/* dummy names for coefficients */
			foreach  scr of global MPROBIT_CSCRS {
				local robnms `"`robnms' `scr':_cons"'
			}
			local i = 0
			local cscrs `"$MPROBIT_CSCRS"'
			forvalues j = 1/`nalt' {
				if (`j' == `ibase') continue
				if `i' == 0 {
					local i = $MPROBIT_NCASECOEF
					continue
				}
				foreach scr of global MPROBIT_CSCRS {
					tempvar c`++i'
					local robnms `"`robnms' `c`i'':_cons"'
					local cscrs `"`cscrs' `c`i''"'
					qui gen double `c`i'' = `scr' if `altern'==`j'
					qui replace `scr' = 0 if `altern' == `j'
					qui by `case': replace `c`i'' = sum(`c`i'')
					qui by `case': replace `c`i'' = `c`i''[_N]
				}
			} 
			foreach scr of global MPROBIT_CSCRS {
				qui replace `scr' = 0 if `altern'==`ibase'
				qui by `case': replace `scr' = sum(`scr')
				qui by `case': replace `scr' = `scr'[_N]
			}
		}
		if `ncov' > 0 {
			/* dummy names for coefficients */
			foreach  scr of global MPROBIT_VSCRS {
				local robnms `"`robnms' `scr':_cons"'
			}
		}
		matrix colnames `V' = `robnms'
		matrix rownames `V' = `robnms'
		_robust $MPROBIT_ASCRS `cscrs' $MPROBIT_VSCRS `iff' [`weight'`exp'], ///
			`clopt2' variance(`V')

		local nms : colfullnames `b'
		matrix rownames `V' = `nms'
		matrix colnames `V' = `nms'

		local nc = r(N_clust)
		ereturn repost b=`b' V=`V' 
		ereturn local chi2
		ereturn local p
		if "`cluster'" != "" {
			ereturn scalar N_clust = `nc'
			ereturn scalar df_r = `nc' - 1
			ereturn local clustvar "`cluster'"
		}
		if `nattrib'>0 | $MPROBIT_NCASECOEF>0 {
			if `nattrib' > 0 {
				qui test [`alternatives']
				local accum a
			}
			if $MPROBIT_NCASECOEF-$MPROBIT_CONST > 0 {
				forvalues i=1/`nalt' {
					if (`i' == `ibase') continue
					local eq : word `i' of $MPROBIT_ALTEQS
					qui test [`eq'], `accum'
					local accum a
				}
			}
			ereturn scalar df_m = r(df)
			local dfr = `nattrib'+($MPROBIT_NCASECOEF-$MPROBIT_CONST)*(`nalt'-1)
			if "`cluster'" != "" {
				ereturn local chi2type
				if (r(df)==`dfr') ereturn scalar F = r(F)
				else ereturn scalar F = .
			}
			else {
				if (r(df)==`dfr') ereturn scalar chi2 = r(chi2)
				else ereturn scalar chi2 = .
			}
			if (r(df)==`dfr') ereturn scalar p = r(p)
		}
		ereturn local vcetype "Robust"
		if ("`weight'"=="pweight") ereturn local wtype "pweight"
	}
	else if "`vce0'" == "opg" {
		ereturn local vcetype "OPG"
		ereturn local ml_method "d1"
	}

	tempname varstr varfix
	matrix `varstr' = vecdiag(`corstr')
	matrix `varfix' = vecdiag(`corfix')
	forvalues i=`nalt'(-1)1 {
		if `i'<=rowsof(`corstr')  {
			matrix `corstr'[`i',`i'] = .
			matrix `corfix'[`i',`i'] = .
		}
		if ("`altlevels'"!="") local alt`i' = `altlevels'[`i',1]
		else local alt`i' = `i'
		if ("`altlabels'"!="") local alt`i' : label `altlabels' `alt`i''

		ereturn local alt`i' `"`alt`i''"'
	}
	local names : rownames `corstr'
	matrix colnames `varstr' = `names'
	matrix colnames `varfix' = `names'
	ereturn matrix corpattern = `corstr'
	ereturn matrix stdpattern = `varstr'
	ereturn matrix corfixed = `corfix'
	ereturn matrix stdfixed = `varfix'

	AlternStats `case' `altern' `choice' `wtoptinit', nalt(`nalt') altlevels(`altlevels')
	tempname stats
	matrix `stats' = r(stats)
	ereturn matrix stats = `stats'

	ereturn scalar N_case = e(N)
	if "`weight'" == "fweight" {
		tempvar fw 
		qui gen `fw' `exp'
		qui summarize `fw' 
		ereturn scalar N = r(sum) 
	}
	else ereturn scalar N = _N
	ereturn scalar alt_min = `casestat'[1,1]
	ereturn scalar alt_avg = `casestat'[2,1]
	ereturn scalar alt_max = `casestat'[3,1]
	ereturn scalar cholesky = `cholesky'
	ereturn scalar fullcov = `fullcov'
	ereturn scalar structcov = `structcov'
	ereturn scalar k_indvars = `nattrib'
	ereturn scalar k_ind2vars = $MPROBIT_NCASECOEF-$MPROBIT_CONST 
	ereturn scalar const = $MPROBIT_CONST
	ereturn scalar k_alt = `nalt'
	ereturn scalar i_base = `ibase'
	ereturn scalar i_scale = `iscale'

	if `nalt' > 2 {
		ereturn scalar mc_points = `intpoints'
		ereturn scalar mc_antithetics = `antithetics'
		if (`imeth'!=3) ereturn scalar mc_burn = `intburn'
	}
	ereturn scalar k_rho = `ncor'
	ereturn scalar k_sigma = `nvar'

	ereturn local indvars  `"`:list retok attribs'"'
	ereturn local ind2vars `"`casevars'"'
	ereturn local alteqs `"$MPROBIT_ALTEQS"'
	ereturn local altvar  `"`alternatives'"'
	ereturn local casevar  `"`case'"'

	if `nalt' > 2 {
		if (`imeth'==1) ereturn local mc_method "Halton"
		else if (`imeth'==2) ereturn local mc_method "Hammersley"
		else {
			ereturn local mc_method "uniform random"
			ereturn local mc_seed `"$MPROBIT_SEED"'
		}
	}
	if ("`correlation'" == "indep")	ereturn local correlation "independent"
	else if ("`correlation'" == "exch") ereturn local correlation "exchangeable"
	else if ("`correlation'" == "unstruct")	ereturn local correlation "unstructured"
	else if ("`cormattype'" == "pattern") ereturn local correlation "pattern matrix"
	else ereturn local correlation "fixed parameter matrix"

	if ("`stddev'" == "heter") ereturn local stddev "heteroskedastic"
	else if ("`stddev'" == "homo") ereturn local stddev "homoskedastic"
	else if ("`varmattype'" == "pattern") ereturn local stddev "pattern matrix"
	else ereturn local stddev "fixed parameter matrix"

	ereturn scalar ic = `ic'
	ereturn matrix ilog = `ilog'

	ereturn local k_dv 

	ereturn local title  "Alternative-specific multinomial probit"
	ereturn local predict asmprobit_p
	ereturn local estat_cmd asmprobit_estat
	ereturn local mfx_dlg asmprobit_mfx
	ereturn local cmd2 asmprobit
	ereturn local cmd asmprobit

	restore
	ereturn repost, esample(`touse')

	Replay, `diopts'
end

program Replay
	syntax [, Level(cilevel)]

	local ncov = e(k_rho)+e(k_sigma)
	local ncasecf : word count `"`e(ind2vars)'"'
	local const = `e(const)' != 0
	local ncasecf = `ncasecf' + `const'

	local noheader noheader 
	local crtype = upper(substr(`"`e(crittype)'"',1,1)) + ///
		substr(`"`e(crittype)'"',2,.)
	di _n in gr `"`e(title)'"' _col(48) "Number of obs" _col(67) "= " in ye ///
	 %10.0g e(N)
	di in gr `"Case variable: `=abbrev("`e(casevar)'",24)'"' _col(48) 	///
	 "Number of cases" _col(67) "= " in ye %10.0g e(N_case) _n
	local altvar `=abbrev("`e(altvar)'",17)'
	di in gr `"Alternative variable: `altvar'"' _col(48) 			///
	 "Alts per case: min = " in ye %10.0g e(alt_min) _n _col(63) in gr 	///
	 "avg = " in ye %10.1f e(alt_avg) _n _col(63) in gr "max = " 		///
	 in ye %10.0g e(alt_max)
	local lencr = length("`e(crittype)'")
	if ("`e(mc_method)'" != "") {
		local sk = `lencr'-length("`e(mc_method)'") - 8
		di in smcl in gr "Integration sequence:" _skip(`sk') "`e(mc_method)'" 
		local sk = `lencr'-21
		local intpoints `""Integration points:" _skip(`sk') in ye %15.0g `=e(mc_points)'"'
	}
	else local intpoints _n

	if "`e(chi2type)'" == "Wald" {
		local stat chi2
        	local cfmt=cond(e(chi2)<1e+7,"%10.2f","%10.3e")
       		if e(chi2) >= . {
			local h help j_robustsingular:
			di in gr `intpoints' in gr _col(51) ///
			 "{`h'Wald chi2(`e(df_m)'){col 67}= }" in ye `cfmt' e(chi2)
       		}
        	else {
			di in gr `intpoints' in gr _col(51) 			    ///
			 "`e(chi2type)' chi2(" in ye "`e(df_m)'" in gr ")" _col(67) ///
			 "= " in ye `cfmt' e(chi2)
       		}
	}
	else {
		/* F statistic from test after _robust */
		local stat F
		local cfmt=cond(e(F)<1e+7,"%10.2f","%10.3e")
		if e(F) < . {
			di in gr `intpoints' in gr _col(51) 			///
			 "F(" in ye %3.0f e(df_m) in gr "," in ye %6.0f e(df_r) ///
			 in gr ")" _col(67) "= " in ye `cfmt' e(F)
		}
		else {
			local dfm = e(df_m)
			local dfr = e(df_r)
			local h help j_robustsingular:

			di in gr `intpoints' in gr _col(51) ///
			 "{`h'F( `dfm', `dfr')}" _col(67) "{`h'=          .}"
		}
	}
	di in gr "`crtype' = " in ye %10.0g e(ll) _col(51) in gr "Prob > `stat'" ///
	 _col(67) "= "  in ye %10.4f e(p) _n

	local coleq : coleq e(b)
	local coleq : list uniq coleq
	_choice_table, begin level(`level')
	if e(k_indvars) {
		if e(const)+e(k_ind2vars)+`ncov' == 0 {
			_choice_table `e(altvar)', equation level(`level') nosep
		}
		else {
			_choice_table `e(altvar)', equation level(`level') 
		}
	}
	if e(const) + e(k_ind2vars) > 0 {
		forvalues i=1/`=e(k_alt)' {
			if (`i'==`=e(k_alt)' & `ncov'==0) local nosep nosep
			local altid : word `i' of `e(alteqs)'
			if `i' == e(i_base) {
				_choice_table `altid', comment("  (base alternative)") `nosep'
			}
			else {
				_choice_table `altid', equation level(`level') `nosep'
			}
		}
	}

	if `ncov' > 0 {
		Replay1 `level'
	}	
	_get_diparmopts, bottom execute
	if e(k_alt) > 2 {
		local i = `e(i_base)'
		if (wordcount("`e(alt`i')'") > 1) local base `""`=abbrev(`"`e(alt`i')'"',17)'""'
		else local base `"`=abbrev(`"`e(alt`i')'"',17)'"'
        	di in gr `"{p}(`altvar'=`base' is the alternative normalizing location){p_end}"'
		if (`e(i_scale)' < .) {
			local i = `e(i_scale)'
			if (wordcount("`e(alt`i')'") > 1) local scale `""`=abbrev(`"`e(alt`i')'"',17)'""'
			else local scale `"`=abbrev(`"`e(alt`i')'"',17)'"'
		        di in gr `"{p}(`altvar'=`scale' is the alternative normalizing scale){p_end}"'
		}
	}
	ml_footnote
end

program Replay1
	args level

	local nalt = e(k_alt)
	if `e(cholesky)' {
		local rii lnl
		local labii l
		local rij l
	}
	else {
		local rii lnsigma
		local labii sigma
		local rij atanhr
	}
	if e(k_sigma) > 0 {
		if !e(fullcov) {
			forvalues i = 1/`e(k_sigma)' {
				_diparm lnsigmaP`i', level(`level')
			}
		}
		else {
			forvalues i = 1/`=colsof(e(stdpattern))' {
				if (el(e(stdpattern),1,`i') >= .) continue
	
				if e(cholesky) local ii `i'_`i'
				else local ii `i'
	
				_diparm `rii'`ii', level(`level')
			}
		}
		if (e(k_rho)>0 | `e(cholesky)'==0) ///
			_get_diparmopts, diparmopts(diparm(__sep__)) execute
	}
	if e(k_rho) > 0 {
		if !e(fullcov) {
			forvalues i=1/`e(k_rho)' {
				_diparm atanhrP`i', level(`level')
			}
		}
		else {
			forvalues j=1/`=rowsof(e(corpattern))' {
				local j1 = `j'+1
				forvalues i=`j1'/`=rowsof(e(corpattern))' {
					if e(fullcov) local ij `i'_`j'
					else local ij = el(e(corpattern),`i',j')
					if (el(e(corpattern),`i',`j') >= .) continue
					_diparm `rij'`ij', level(`level')
				}
			}
		}
		if (`e(cholesky)'==0) ///
			_get_diparmopts, diparmopts(diparm(__sep__)) execute
	}
	if (e(cholesky) != 0) exit
	if !e(fullcov) local P P
	if e(k_sigma) > 0 {
		forvalues i = 1/`nalt' {
			if (el(e(stdpattern),1,`i') >= .) {
				if (`i'==e(i_base)) local cmnt "(base alternative)"
				else if (`i'==e(i_scale)) local cmnt "(scale alternative)"
				else local cmnt "(fixed)"

				_choice_table sigma`i', ///
					value(`=el(e(stdfixed),1,`i')') ///
					comment("  `cmnt'") nosep
			}
			else {
				if e(fullcov) local ii `i'
				else local ii = el(e(stdpattern),1,`i')
	
				_diparm lnsigma`P'`ii', ///
				level(`level') exp label(sigma`i')
			}
		}
	}
	if e(k_rho) > 0 {
		if (e(k_sigma) > 0) ///
			_get_diparmopts, diparmopts(diparm(__sep__)) execute
		forvalues j=1/`nalt' {
			local j1 = `j'+1
			forvalues i=`j1'/`nalt' {
				if e(fullcov) local ij `i'_`j'
				else local ij = el(e(corpattern),`i',`j')

				if (el(e(corpattern),`i',`j') >= .) {
					if `i'== e(i_base)|`j' == e(i_base) {
						if (el(e(corfixed),`i',`j')==0) ///
							continue
						else local cmnt "(base alternative)"
					}
		    			else local cmnt "(fixed)"

					_choice_table rho`i'_`j', 	      ///
					    value(`=el(e(corfixed),`i',`j')') ///
					    comment("  `cmnt'") nosep
				}
				else {
					_diparm atanhr`P'`ij', level(`level') ///
						function(tanh(@)) 	      ///
						derivative((1-tanh(@)^2))     ///
						label(rho`i'_`j')
				}
			}
		}
	}
end

program PostError
	syntax , [SCore(string)]
 
	if "`score'"!="" {
		di as error "option scores() not allowed"
		exit 198
	}
end
  
program ParseTech, sclass
	args technique initbhhh vce

	if "`vce'"!="" & "`vce'"!="oim" & "`vce'"!="opg" & "`vce'"!="native" {
		di as error "{p}vce must be either oim, opg, robust, bootstrap, " ///
		 "or jackknife{p_end}"
		exit 198
	}
	if "`initbhhh'"!="" & `initbhhh'<0 {
		di as error "initbhhh must be greater than 0"
		exit 198
	}

	tokenize `technique'

	local d1 d1
	local vce0 
	local tech `technique'
	if "`2'" == "" {
		if "`1'" == "" {
			local tech bfgs
			local technique bfgs
		}
		if "`vce'" == "native" {
			if ("`tech'" == "nr")	local vce oim
			else if ("`tech'" == "bhhh") local vce opg
		}
		local vce0 = "`vce'"
		if "`tech'" == "bhhh" {
			local initbhhh = 0
			/* must trick -ml- and let the likelihood evaluator compute the */
			/* outer product gradient approximation to the negative Hessian */
			local d1 d2
			local tech nr

			/* detect if a extra call to ml is needed to get the desired vce */
			if "`vce'"=="oim" {
				local tech2 nr
				local vce2  oim
				local d21   d1
			}
			else { // "`vce'"=="opg" | "`vce'"=="native"
				local vce oim
				local vce0 opg
			} 
			else local vce0 opg
		}
		else if "`vce'" == "opg" {
			/* detect if a extra call to ml is needed to get the desired vce */
			local tech2 nr
			local vce2  oim
			local d21   d2
		}
		if `initbhhh' > 0 {
			if ("`tech'" == "nr") local techmeth Newton-Raphson
			else local techmeth = upper("`technique'")
		}
		sreturn local techswitch = 0
	}
	else {
		local initbhhh = 0
		local i = 0
		while "``++i''" != "" {
			if "``i''" == "bhhh" {
				di as error "{p}technique(`technique') is invalid; " ///
				 "use the initbhhh({it:#}) option{p_end}"
				exit 184
			}
		}
		local tech2 nr
		if "`vce'"=="" | "`vce'"=="oim" {
			local vce2  oim
			local d21   d1
			local vce0  oim 
		}
		else {
			local vce2  oim
			local d21   d2
			local vce0  opg
		}
		sreturn local techswitch = 1
	}
	sreturn local technique `"`technique'"'
	sreturn local tech `"`tech'"'
	sreturn local d1 "`d1'"
	sreturn local vce0 "`vce0'"
	sreturn local vce "`vce'"
	sreturn local tech2 "`tech2'"
	sreturn local vce2 "`vce2'"
	sreturn local d21 "`d21'"
	sreturn	local initbhhh = `initbhhh'
	sreturn local techmeth "`techmeth'"
end

program GetModel, rclass
	syntax varlist(max=1) [if] [fw pw iw],	///
		alternatives(varname) 		///
		altname(string)			///
		ibase(integer) [		///
		case(varname)			///
		attribs(varlist) 		///
		noconstant			///
		casevars(varlist) 		///
		constraints(numlist) ]

	tempname from caselvls b ind
	local np = 0
	local const = ("`constant'" == "")
	local nccf : word count `casevars'
	local nacf : word count `attribs'

	 _labels2names `alternatives', stub(_alt_) noint
	local alteqs `"`s(names)'"'
	local nalt = `s(n_cat)'

	local ncoef = 0

	if `nacf' > 0 {
		local altmodel `"(`altname':`varlist'="'
		foreach attrib of local attribs {
			local cfnms `"`cfnms' `altname':`attrib'"'
			local altmodel `"`altmodel' `attrib'"'
			local `++ncoef'
		}
		local altmodel `"`altmodel',nocons)"'
	}
	if `const' | `nccf'>0 {
		local k = 0

		forvalues i=1/`nalt' {
			if (`i'==`ibase') continue

               		local ai : word `i' of `alteqs'
			local `++k'
			if (`k'==1 & `nacf'==0)	local casemodels `"`casemodels' (`ai':`varlist'="'
			else local casemodels `"`casemodels' (`ai':"'

			if `nccf' > 0 {
				foreach cv of local casevars {
					local cfnms `"`cfnms' `ai':`cv'"'
					local casemodels `"`casemodels' `cv'"'
					local `++ncoef'
				}
			}
			if `const' {
				local cfnms `"`cfnms' `ai':_cons"'
				local casemodels `"`casemodels')"'
				local `++ncoef'
			}
			else local casemodels `"`casemodels',nocons)"'
		}
	}
	return local ncoef = `ncoef'
	return local altmodel `"`altmodel'"'
	return local casemodels `"`casemodels'"'
	return local alteqs `"`alteqs'"'
	return local ncasecf = `nccf'+`const'
	return local const = `const'
	return local rcfnms `"`cfnms'"'
end

program InitMProbit, rclass
	syntax varlist(max=1) [if] [fw pw iw],	///
		case(varname)			///
		alternatives(varname) 		///
		ibase(integer) 			///
		coefnames(string)[		///
		attribs(varlist) 		///
		noCONSTant			///
		casevars(varlist) 		///
		cmat(string) ]

	tempname from b 
	local const = ("`constant'" == "")
	local nccf : word count `casevars'
	local nacf : word count `attribs'

	 _labels2names `alternatives', stub(_alt_) noint
	local altlabels `"`s(names)'"'
	local nalt = `s(n_cat)'
		
	if `nacf' == 0 {
		if (`const'==0) local noconst noconstant
		/* must generate mlogit coefficient names in case it drops variables due */
		/* to collinearity */
		tempvar resp 
		tempname lab
		qui gen `resp' = `alternatives'
		forvalues i=1/`nalt' {
               		local ai : word `i' of `altlabels'

			label define `lab' `i' `ai', add
			if (`i'==`ibase') continue

			if `nccf' > 0 {
				foreach cv of local casevars {
					local initcf `"`initcf' `ai':`cv'"'
				} 
			}
			if (`const') local initcf `"`initcf' `ai':_cons"'
		}
		label values `resp' `lab'
		if (`const'==0) local noconst noconst
		if "`cmat'" != "" {
			local n: word count `initcf'
			matrix colnames `cmat' = `initcf' c`++n'
			local copt constraints(`cmat')
		}
		cap qui mlogit `resp' `casevars' [`weight'`exp'] `if', `noconst' ///
			ltol(0.0001) tol(0.001) base(`ibase') `copt' iterate(50)

		local rc = _rc
		if `rc' == 503 {
			/* constraints on collinear variables */
			cap qui mlogit `resp' `casevars' [`weight'`exp'] `if', `noconst' ///
				ltol(0.0001) tol(0.001) base(`ibase')  iterate(50)
		}
		else if `rc' > 0 {
			di as error "initial estimates failed"
			error `rc' 
		}
		matrix `b' = e(b)
	}
	else {
		/* must generate clogit coefficient names in case it drops variables due */
		/* to collinearity */
		foreach attrib of local attribs {
			local initcf `"`initcf' `varlist':`attrib'"'
		}
		if `const' | `nccf'>0 {
			forvalues i=1/`nalt' {
				if (`i'==`ibase') continue

				tempvar a`i'
				gen byte `a`i'' = (`alternatives'==`i')
				if `nccf' > 0 {
					foreach cv of local casevars {
						tempvar `cv'`ai'
						gen double ``cv'`ai'' = `a`i''*`cv'
						local csvars `"`csvars' ``cv'`ai''"'
						local initcf `"`initcf' `varlist':``cv'`ai''"'
					} 
				}
				if `const' {
					local csvars `"`csvars' `a`i''"'
					local initcf `"`initcf' `varlist':`a`i''"'
				}
			}
		}
		if "`cmat'" != "" {
			local n: word count `initcf'
			matrix colnames `cmat' = `initcf' c`++n'
			local copt constraints(`cmat')
		}
		cap qui clogit `varlist' `attribs' `csvars' [`weight'`exp'], group(`case') ///
			ltol(0.0001) tol(0.001) nrtol(0.01) `copt' iterate(50)  

		local rc = _rc
		if `rc' == 503 {
			/* constraints on collinear variables */
			qui clogit `varlist' `attribs' `csvars' [`weight'`exp'], group(`case') ///
			   ltol(0.0001) tol(0.001) nrtol(0.01) iterate(50)  
		}
		else if `rc' > 0 {
			di as error "initial estimates failed"
			error `rc' 
		}
		matrix `b' = e(b)*sqrt(6)/c(pi)
	}
	if (e(converged)==0) di in gr "note: computation of initial estimates failed"
	local ncoef : word count `coefnames'
	if colsof(`b') < `ncoef' {
		di in gr "{p 0 7}note: two or more of the variables are collinear; " ///
	 	 "convergence may not be achieved"

		matrix `from' = J(1,`ncoef',0)
		local ndrop = 0
		forvalues i=1/`ncoef' {
			local icf : word `i' of `initcf'
			local k = colnumb(`b',`"`icf'"')
			
			if (`k'<.)  matrix `from'[1,`i'] = `b'[1,`k']
		}
	}
	else matrix `from' = `b'

	matrix colnames `from' = `coefnames' 
	return matrix from = `from'
end

program MatrixName, rclass
	cap syntax namelist(min=1 max=2), type(string) 

	if _rc > 0 {
		di as error "correlation() or stddev() incorrectly specified"
		exit 198
	}

	gettoken spec name : namelist

	local lspec = length("`spec'")
	if "`spec'" == substr("pattern",1,max(3,`lspec')) {
		return local matspec pattern
	}
	else if "`spec'" == substr("fixed",1,max(3,`lspec')) {
		return local matspec fixed
	}
	else if "`type'" == "correlation" {
		di as error "{p}correlation() must specify either independent, " ///
		 "exchangeable, unstructured (default), or a matrix "            ///
		 "specifying the correlation pattern or fixed parameters{p_end}" 
		exit 198
	}
	else {
		di as error "{p}stddev() must specify either heteroskedastic, "  ///
		 "homoskedastic, or a matrix specifying the standard deviation " ///
		 "pattern or fixed parameters{p_end}"
		exit 198
	}
	if "`name'" == "" {
		di as error "{p}a matrix name must follow the matrix specification " ///
		 "`spec' in `type'(){p_end}"
		exit 198
	}
	if _rc != 0 {
		di as error "syntax error specifying `type'()"
		exit _rc
	}
	cap qui confirm matrix `name'
	if _rc != 0 {
		di as error "matrix `name' specified in `type'() does not exist"
		exit 111
	}
	return local name `name'
end

program VParStripe, rclass
	args cholesky

	local nvcf = 0
	if $MPROBIT_FULLCOV {
		local dim = $MPROBIT_NALT
		if `cholesky' {
			local rii lnl
			local rij l
			if !$MPROBIT_STRUCTCOV {
				local `--dim'
			}
		}
		else {
			local rii lnsigma
			local rij atanhr
		}
		forvalues j=1/`dim' {
			local aj = $MPROBIT_CORSTRUCT[`j',`j']
			if `aj' < . {
				if (`cholesky') local jj `j'_`j'
				else local jj `j'
				local thetas `"`thetas' /`rii'`jj'"'
				local cfnms `"`cfnms' `rii'`jj':_cons"'
				local `++nvcf'
			}
		}
		forvalues j=1/`dim' {
			local j1 = `j'+1
			forvalues i=`j1'/`dim' {
				local ai = $MPROBIT_CORSTRUCT[`i',`j']
				if `ai' < . {
					local thetas1 `"`thetas1' /`rij'`i'_`j'"'
					local cfnms1 `"`cfnms1' `rij'`i'_`j':_cons"'
					local `++nvcf'
				}
			}
		}
	}
	else {
		forvalues j=1/$MPROBIT_NVAR {
			local thetas `"`thetas' /lnsigmaP`j'"'
			local cfnms `"`cfnms' lnsigmaP`j':_cons"'
		}
		forvalues i=1/$MPROBIT_NCOR {
			local thetas1 `"`thetas1' /atanhrP`i'"'
			local cfnms1 `"`cfnms1' atanhrP`i':_cons"'
		}
		local nvcf = $MPROBIT_NVAR+$MPROBIT_NCOR
	}
	if `"`thetas1'"' != "" {
		local thetas `"`thetas' `thetas1'"'
		local cfnms `"`cfnms' `cfnms1'"'
	}
	return local nvcf `nvcf'
	return local thetas `"`thetas'"'
	return local vcfnms `"`cfnms'"'
end

program Reparameterize, rclass
	args b cfnms order T

	tempname R
	matrix `R' = I($MPROBIT_NALT)

	/* cholesky factored covariance */
	local rc = 0
	forvalues j=1/$MPROBIT_NALT {
		local aj = $MPROBIT_CORSTRUCT[`j',`j']
		if `aj' < . {
			local k = colnumb("`b'","lnl`j'_`j':") 
			if `k' >= . {
				di as error "could not find coefficient associated " ///
				 "with parameter lnl`j'_`j':_cons"
				local rc = 498
				continue, break
			}
			matrix `R'[`j',`j'] = exp(`b'[1,`k']) 
		}
		local j1 = `j'+1
		forvalues i=`j1'/$MPROBIT_NALT {
			local ai = $MPROBIT_CORSTRUCT[`i',`j']
			if (`ai' >=.) continue

			local k =  colnumb("`b'","l`i'_`j':")
			if `k' >= . {
				local rc = 498
				di as error "could not find coefficient associated " ///
				 "with parameter l`i'_`j':_cons"
				continue, break
			}
			matrix `R'[`i',`j'] = `b'[1,`k']
		}
		if (`rc'>0) continue, break
	}
	if (`rc'>0) exit `rc'
	matrix `R' = `R'*`R''
	local reorder = 0
	local j = 0 
	while `++j'<$MPROBIT_NALT & !`reorder' {
		local reorder = (`order'[`j',1] != $MPROBIT_ORDER[`j',1])
	}
	if `reorder' {
		/* put variance-covariances back into the original alternative 	*/
		/*  order before permuting for base and scale alternatives	*/
		local names : rownames $MPROBIT_CORSTRUCT
		matrix rownames `R' = `names'

		PivotMat "$MPROBIT_CORSTRUCT" "`order'" 0 
		PivotMat "$MPROBIT_CORFIXED" "`order'" 0
		PivotMat "`R'" "`order'" 0
		global MPROBIT_ORDER `order'
	}
	local kv = 0
	local kc = 0

	local k : word count `cfnms'
	forvalues j=1/$MPROBIT_NALT {
		local aj = $MPROBIT_CORSTRUCT[`j',`j']
		if `aj' < . {
			matrix $MPROBIT_CORSTRUCT[`j',`j'] = `++kv'
			matrix `b'[1,`++k'] = log(sqrt(`R'[`j',`j']))
			local cfnms `"`cfnms' lnsigma`j':_cons"'
			local thetas `"`thetas' /lnsigma`j'"'
		}
	}
	forvalues j=1/$MPROBIT_NALT {
		local j1 = `j'+1
		forvalues i=`j1'/$MPROBIT_NALT {
			local ai = $MPROBIT_CORSTRUCT[`i',`j']
			if (`ai' >=.) continue

			matrix `b'[1,`++k'] = atanh(`R'[`i',`j']/sqrt(`R'[`i',`i']*`R'[`j',`j']))

			matrix $MPROBIT_CORSTRUCT[`i',`j'] = `++kc'
			matrix $MPROBIT_CORSTRUCT[`j',`i'] = `kc'
			local cfnms `"`cfnms' atanhr`i'_`j':_cons"'
			local thetas `"`thetas' /atanhr`i'_`j'"'
		}
	}
	if `reorder' {
		GenTmat "$MPROBIT_NVAR" "$MPROBIT_NCOR" "$MPROBIT_CORSTRUCT" 
		matrix $MPROBIT_TMAT = r(T)
	}
	matrix colnames `b' = `cfnms' 
	return local thetas `"`thetas'"' 
end

program PivotMat, rclass
	args mat order renumber

	local nalt = rowsof(`order')
	tempname mat1 ip
	matrix `mat1' = J(`nalt',`nalt',.)
	matrix `ip' = J(`nalt',1,.)

	local levels : rownames `mat'

	forvalues i=1/`nalt' {
		local ci = `order'[`i',1]
		local olevels `"`olevels' `ci'"'

		local ci : word `i' of `levels'
		local j = 0
		local found = 0
		while `found'==0 & `j'<`nalt' {
			cap local found = (`ci' == `order'[`++j',1])
			if (_rc > 0) continue, break
		}
		if `found' == 0 {
			di as error "internal error pivoting matrix" 
			exit 498
		}
		matrix `ip'[`i',1] = `j'
		forvalues k=1/`nalt' {
			matrix `mat1'[`j',`k'] = `mat'[`i',`k']
		}
	}
	matrix `mat' = `mat1'
	forvalues i=1/`nalt' {
		local k = `ip'[`i',1]
		forvalues j=1/`nalt' {
			matrix `mat1'[`j',`k'] = `mat'[`j',`i']
		}
	}
	matrix `mat' = `mat1'
	matrix rownames `mat' = `olevels'
	matrix colnames `mat' = `olevels'
	
	if `renumber' {
		/* used for corpattern and corfixed matrices */
		local kd = 0
		local kl = 0
		forvalues j=1/`nalt' {
			if `mat'[`j',`j'] < . {
				matrix `mat'[`j',`j'] = `++kd'
			}
			local j1 = `j'+1
			forvalues i=`j1'/`nalt' {
				if `mat'[`i',`j'] < . {
					matrix `mat'[`i',`j'] = `++kl'
					matrix `mat'[`j',`i'] = `kl'
				}
			}
	
		}
	}

end

program CopyVarStruct, rclass
	args corstr corfix varstr varmat vartype ibase ifixed 

	local levels : rownames `corstr'
	local nalt : word count `levels'

	if (rowsof(`varstr')==`nalt' & colsof(`varstr')==1) matrix `varstr' = `varstr''
	
	if rowsof(`varstr')!=1 | colsof(`varstr')!=`nalt' {
		di as error "matrix specified in stddev() must be 1 by `nalt'"
		exit 198
	}
	matrix colnames `varstr' = `levels'
	local nvar = 0
	if "`vartype'" == "pattern" {
		tempname v
		matrix `v' = J(`nalt',1,0) 
	}
	forvalues i=1/`nalt' {
		if "`vartype'" == "pattern" {
			if (`varstr'[1,`i'] == 0) matrix `varstr'[1,`i'] = .
			local k = `varstr'[1,`i'] 
			matrix `corstr'[`i',`i'] = `k'
			if `k' >= . {
				/* fixed to 1 */
				if (`ibase'>=. & `ifixed'!=`i') local ibase = `i'
				else if (`ifixed'>=. & `ibase'!=`i') local ifixed = `i'

				matrix `corfix'[`i',`i'] = 1
			}
			else {
				if `k'<1 | `k'>`nalt' {
					di as error "{p}variance parameters identified in " ///
					 "`varmat' must be consecutive starting from 1 "    ///
					 "and cannot exceed `nalt'{p_end}"
					exit 459
				}
				matrix `v'[`k',1] = `v'[`k',1] + 1
				if (`k' > `nvar') local nvar = `k'
				/* free parameter */
				matrix `corfix'[`i',`i'] = .
			}
			
		}
		else { /* "`vartype'" == "fixed" */
			local val = `varstr'[1,`i']
			matrix `corfix'[`i',`i'] = `val'
			if `val' < . {
				if `val' <= 0.0 {
					di as error "{p}fixed standard deviation "    ///
					 "`varmat'[1,`i'] = " %10.0g `val' " is not " ///
					 "valid {p_end}"
					exit 411
				}
				/* fixed to a user specified constant */
				if (`ibase'>=. & `ifixed'!=`i') local ibase = `i'
				else if (`ifixed'>=. & `ibase'!=`i') local ifixed = `i'

				matrix `corstr'[`i',`i'] = .
			}
			else {
				/* free parameter, add to pattern matrix */
				matrix `corstr'[`i',`i'] = `++nvar'
			}
		}
	}
	if "`vartype'" == "pattern" {
		forvalues i=1/`nvar' {
			if `v'[`i',1] == 0 {
				di as error "{p}variance parameters identified in "  ///
				 "`varmat' must be consecutive starting from 1 and " ///
				 "cannot exceed `nalt'{p_end}"
				exit 459
			}
		}
	}
	return local ibase `ibase'
	return local ifixed `ifixed'
	return local nvar `nvar'
end

program CorStruct, rclass
	args nalt correlation stddev ibase ifixed scale structcov varmat vartype

	tempname corstr corfix order
	matrix `corstr' = J(`nalt',`nalt',.)
	matrix `corfix' = I(`nalt')*`scale'
	matrix `order' = J(`nalt',1,1)
	local levels  "1"
	forvalues i=2/`nalt' { 
		matrix `order'[`i',1] = `i'
		local levels `"`levels' `i'"'
	}
	matrix rownames `corstr' = `levels'
	matrix colnames `corstr' = `levels'
	matrix rownames `corfix' = `levels'
	matrix colnames `corfix' = `levels'

	if `varmat' {
		tempname varstr 
		matrix `varstr' = `stddev'
		local varmat = "`stddev'"
		local stddev = "user" 

		CopyVarStruct "`corstr'" "`corfix'" "`varstr'" "`varmat'" "`vartype'" ///
			`ibase' `ifixed' 
		local ibase = `r(ibase)'
		local ifixed = `r(ifixed)'
		local nvar = `r(nvar)'

		if `ibase' >= . {
			di as error "{p}could not find an appropriate base alternative; " ///
			 "at least two standard deviations must be fixed, or use the "    ///
			 "basealternative() option{p_end}"
			exit 498
		}
		else if `ifixed' >= . {
			di in gr "{p 0 7}note: could not find an appropriate scale " ///
			 "alternative; the model may not be scaled{p_end}"
		}
	}
	else {
		if `ibase' >= .  {
			loca ibase = 1
			if (`ibase' == `ifixed') local ibase = 2
		}
		if `ifixed' >= . {
			local ifixed = 1
			if (`ifixed' == `ibase') local ifixed = 2
		}
	}

	local alpha = 1
	local sigma = 1

	if "`correlation'" == "indep" {
		forvalues i=1/`nalt' {
			if (`i'==`ibase' | `i'==`ifixed') continue
			if "`stddev'" == "heter" {
				matrix `corstr'[`i',`i'] = `sigma'
				matrix `corfix'[`i',`i'] = .
				local sigma = `sigma'+1
			}
		}
		return local ncor = 0
		if ("`stddev'"=="user") return local nvar = `nvar'
		else return local nvar = `sigma'-1
	}
	else { /* if `structcov' { */
		local ibs = 0
		local ifx = 0
		forvalues j=1/`nalt' {
			if (`j'==`ibase') continue
			else if "`stddev'"=="heter" & `j'!=`ifixed' {
				matrix `corstr'[`j',`j'] = `sigma'
				matrix `corfix'[`j',`j'] = .
				local sigma = `sigma'+1
			}
			local i1 = `j'+1
			forvalues i=`i1'/`nalt' {
				if `i' != `ibase' { 
					matrix `corstr'[`i',`j'] = `alpha'
					matrix `corstr'[`j',`i'] = `alpha'
					matrix `corfix'[`i',`j'] = .
					matrix `corfix'[`j',`i'] = .
					if ("`correlation'"=="unstruct") local alpha = `alpha'+1
				}
			}
		}
		if ("`correlation'"=="unstruct") local alpha = `alpha'-1
		local sigma = `sigma'-1

		return local ncor = `alpha'
		if ("`stddev'"=="user") return local nvar = `nvar'
		else return local nvar = `sigma'
	}
	return matrix order = `order'
	return local ibase = `ibase'
	return local ifixed = `ifixed'	
	return matrix corstr = `corstr'
	return matrix corfix = `corfix'
end
	
program ValidateCorStruct, rclass
	args corstr nalt cormat vars ibase ifixed cortype bvarmat vartype

	if rowsof(`corstr')!=`nalt' | colsof(`corstr')!=`nalt' {
		di as error "matrix specified in correlation() must be " ///
		 "`nalt' by `nalt'"
		exit 198
	}
	tempname corfix order
	if "`cortype'" == "fixed" {
		matrix `corfix' = `corstr'
		matrix `corstr' = J(`nalt',`nalt',.)
	}
	else {
		matrix `corfix' = J(`nalt',`nalt',0)
	}
	matrix `order' = J(`nalt',1,1)
	matrix `corstr'[1,1] = .
	local levels "1"
	forvalues i=2/`nalt' {
		matrix `order'[`i',1] = `i'
		matrix `corstr'[`i',`i'] = .
		if `corstr'[`i',1] == 0 {
			matrix `corstr'[`i',1] = .
			matrix `corstr'[1,`i'] = .
		}
		local i1 = `i'+1
		forvalues j=`i1'/`nalt' {
			if `corstr'[`j',`i'] == 0 {
				matrix `corstr'[`j',`i'] = .
				matrix `corstr'[`i',`j'] = .
			}
		}
		local levels `"`levels' `i'"'
	}
	matrix rownames `corstr' = `levels'
	matrix colnames `corstr' = `levels'
	matrix rownames `corfix' = `levels'
	matrix colnames `corfix' = `levels'

	if `bvarmat' {
		tempname varstr 
		local varmat "`vars'"
		matrix `varstr' = `vars'
		local vars "user"

		CopyVarStruct "`corstr'" "`corfix'" "`varstr'" "`varmat'" "`vartype'" ///
			`ibase' `ifixed' 
		local nvar = `r(nvar)'
	}
	local ncor = 0
	if "`cortype'" == "pattern" {
		tempname r  
		local n = `nalt'*(`nalt'-1)/2
		matrix `r' = J(`n',1,0)
	}
	tempname nfree
	matrix `nfree' = J(1,`nalt',0)
	forvalues i=1/`nalt' {
		local j1 = `i'+1 
		forvalues j=`j1'/`nalt' {
			if "`cortype'"=="pattern" {
				local k = `corstr'[`j',`i']
				if  `k' < . {
					if `k'<1 | `k'>`n' {
						di as error "{p}correlation parameters identified " ///
						 "in `cormat' must be consecutive starting "   ///
						 "from 1 and cannot exceed `n'{p_end}"
						exit 459
					}
					matrix `r'[`k',1] = `r'[`k',1] + 1
					if  (`k' > `ncor') local ncor = `k'
					matrix `corfix'[`j',`i'] = .
					matrix `nfree'[1,`j'] = `nfree'[1,`j'] + 1
					matrix `nfree'[1,`i'] = `nfree'[1,`i'] + 1
				}
			}
			else {
				local val = `corfix'[`j',`i']
				if `val' < . {
					if `val'<-1 | `val'>1 {
						di as error "{p}fixed correlation identified in " ///
						 "`cormat' must be between -1 and 1 inclusive{p_end}"
						exit 459
					}
				}
				else {
					matrix `corstr'[`j',`i'] = `++ncor'
					matrix `nfree'[1,`j'] = `nfree'[1,`j'] + 1
					matrix `nfree'[1,`i'] = `nfree'[1,`i'] + 1
				}
			}
			/* ensure symmetric, lower triangular determines the content */
			matrix `corstr'[`i',`j'] = `corstr'[`j',`i'] 
			matrix `corfix'[`i',`j'] = `corfix'[`j',`i']
		}
	}
	if "`cortype'" == "pattern" {
		forvalues i=1/`ncor' {
			if `r'[`i',1] == 0 {
				di as error "{p}correlation parameters identified in " ///
				 "`cormat' must be consecutive starting from 1 and "   ///
				 "cannot exceed `n'{p_end}"
				exit 459
			}
		}
	}
	/* try to find suitable base alternative */
	if `ibase' >= . {
		forvalues i=1/`nalt' {
			if `corstr'[`i',`i']>=. & `nfree'[1,`i']==0 {
				local ibase = `i'
				continue, break
			}
		}
		if `ibase' >= . {
			forvalues i=1/`nalt' {
				if `nfree'[1,`i'] == 0 {
					local ibase = `i'
					continue, break
				}
			}
		}
	}
	if `ibase' >= . {
		di as error "{p}could not find an appropriate base alternative; use the " ///
		 "basealternative() option{p_end}"
		exit 498
	}
	/* identify a suitable scale alternative */
	if `ifixed' == . {
		forvalues i=1/`nalt' {
			if `corstr'[`i',`i'] >= . {
				if `ibase' != `i' {
					local ifixed = `i'
					continue, break
				}
			}
		}
	}
	if `ifixed' >= . {
		di in gr "{p 0 7}note: could not find an appropriate scale alternative; " /// 
		 "the model may not be scaled{p_end}"
	}
	else if !`bvarmat' {
		local nvar = 0
		if "`vars'" == "heter" {
			forvalues i=1/`nalt' {
				if `i'!=`ibase' & `i'!=`ifixed' {
					matrix `corstr'[`i',`i'] = `++nvar'
					matrix `corfix'[`i',`i'] = .
				}
				else {
					matrix `corstr'[`i',`i'] = .
					matrix `corfix'[`i',`i'] = 1
				}
			}
		}
		else { /* if "`vars'" == "homo" */
			forvalues i=1/`nalt' {
				matrix `corstr'[`i',`i'] = .
				matrix `corfix'[`i',`i'] = 1
			}
		}
	}
	
	return local ncor = `ncor'
	return local nvar = `nvar'
	return local stddev  "`vars'"
	return local ibase = `ibase'
	return local ifixed = `ifixed'
	return matrix corfix = `corfix'
	return matrix corstr = `corstr'
	return matrix order = `order'
end

program AlternativeIndex, rclass
	args  altlevels nalt altlabels level type

	local index = .
	if "`level'"!="" {
		if "`altlevels'" != "" {
			local i = 0
			while `++i'<=`nalt' & `index'>=. {
				local ialt = `altlevels'[`i',1]
				if (`"`level'"'==`"`ialt'"') local index = `i'
			}
		}
		if `index'>=. & "`altlabels'"!="" {
			local i = 0
			while `++i'<=`nalt' & `index'>=. {
				if "`altlevels'" != "" {
					local labi : label `altlabels' `=`altlevels'[`i',1]'
				}
				else {
					local labi : label `altlabels' `i'
				}
				if (`"`level'"'==`"`labi'"') local index = `i'
			}
		}
		if "`type'"!="" & `index'>=. {
			di as error "{p}`type' `level' is not one of the alternatives; " ///
			 "use {help tabulate##|_new:tabulate} for a list of values{p_end}"
			exit 459
		}
	}
	return local index = `index'
end 

program GenTmat, rclass
	args nv nr corstr

	tempname T
	local nc = `nv'+`nr'
	local m = rowsof(`corstr')
	local np = `m'*(`m'+1)/2

	matrix `T' = J(`np',`nc',0)
	local i = 1
	forvalues j=1/`m' {
		local j2 = `corstr'[`j',`j']
		if (`j2'<.) matrix `T'[`i',`j2'] = 1
		local j1 = `j' + 1
		forvalues k=`j1'/`m' {
			local i = `i'+1
			if `corstr'[`k',`j'] < . {
				local lc = `corstr'[`k',`j']
				local j2 = `nv'+`lc'
				matrix `T'[`i',`j2'] = 1
			}
		}
		local i = `i'+1
	}

	return matrix T = `T'
end

program CheckConstraints, eclass 
	args b nv clist

	local vconstr = 0
	local bconstr = 0
	if `"`clist'"' == "" {
		ereturn local vconstr = `vconstr'
		ereturn local bconstr = `bconstr'
		exit 0
	}
	tempname T a C CV V b1 CB
		
	matrix `b1' = `b'
	matrix `V' = `b''*`b'
	ereturn post `b1' `V' 
	makecns `clist', nocnsnotes r
	if r(k) == 0 {
		ereturn local vconstr = `vconstr'
		ereturn local bconstr = `bconstr'
		exit 0
	}	
	matcproc `T' `a' `C'

	local nr = rowsof(`C')
	local nc = colsof(`C')
	local i1 = `nc'-`nv'-1
	forvalues i=1/`nr' {
		forvalues j=1/`i1' {
			if `C'[`i',`j'] != 0 {
				local `++bconstr'
				matrix `CB' = (nullmat(`CB')\(`C'[`i',1..`i1'],`C'[`i',`nc']))
				continue, break
			}
		}
	}
	ereturn local bconstr = `bconstr'
	if (`bconstr' > 0) ereturn matrix C = `CB'

	if `nv' > 0 {
		local `++i1'
		local `--nc'
		local j = 0

		forvalues i = 1/`nr' {
			forvalues j = `i1'/`nc' {
				if `C'[`i',`j'] != 0 {
					local `++vconstr'
					continue, break
				}
			}
		}
	}

	ereturn local vconstr = `vconstr'
end

program AlternStats, rclass
	syntax varlist [fw iw], nalt(integer) [ altlevels(string) ]

	gettoken case varlist : varlist
	gettoken alternatives varlist : varlist
	gettoken choice varlist : varlist	

	tempvar ix
	tempname stats sum
	matrix `stats' = J(`nalt',4, 0)
	matrix colnames `stats' = level case_present freq_select per_select
	qui gen int `ix' = 0
	scalar `sum' = 0
	forvalues i=1/`nalt' {
		local index `"`index' `i'"'
		if ("`altlevels'" != "") matrix `stats'[`i',1] = `altlevels'[`i',1]
		else matrix `stats'[`i',1] = `i'
		qui replace `ix' = 1 if `alternatives' == `i'
		qui summarize `ix' [`weight'`exp'] 
		matrix `stats'[`i',2] = r(sum)
		qui replace `ix' = 0
		qui replace `ix' = 1 if `alternatives'==`i' & `choice'==1
		qui summarize `ix' [`weight'`exp'] 
		matrix `stats'[`i',3] = r(sum)
		scalar `sum' = `sum'+`stats'[`i',3] 
		qui replace `ix' = 0
	}
	matrix rownames `stats' = `index'
	forvalues i=1/`nalt' {
		matrix `stats'[`i',4] = `stats'[`i',3]/`sum'
	}
	
	return matrix stats = `stats'
end

program InitDifferencedCov, rclass 
	args n scale

	tempname V b
	local n1 = `n'-1
	matrix `V' = J(`n1',`n1',`scale')+I(`n1')*`scale'
	matrix `V' = cholesky(`V')
	local m = `n'*(`n'-1)/2-1
	matrix `b' = J(1,`m',0)
	local k = `n1'-1
	forvalues i=1/`n1' {
		if (`i'>1) matrix `b'[1,`=`i'-1'] = log(`V'[`i',`i'])
		local i1 = `i'+1
		forvalues j=`i1'/`n1' {
			matrix `b'[1,`++k'] = `V'[`j',`i']
		}
	}
	return matrix b = `b'
end

program CheckMemoryForRobust

	local nalt = $MPROBIT_NALT-2
	if (`nalt'<=0) exit

	local i = 0
	forvalues j = 1/`nalt' {
		foreach scr of global MPROBIT_CSCRS {
			tempvar c`++i'
			qui gen double `c`i'' = 0
		}
	} 
end

