*! version 2.0.2  25apr2006
program asmprobit_estat
	version 9

	if "`e(cmd2)'" != "asmprobit" {
		di as err "{help asmprobit##|_new:asmprobit} estimation results not found"
		exit 301
	}

	gettoken sub rest: 0, parse(" ,")

	local lsub = length("`sub'")
	if "`sub'" == substr("covariance",1,max(3,`lsub')) | ///
	   "`sub'" == substr("correlation",1,max(3,`lsub')) {
		local which = substr("`sub'",1,3)
		Covariance `which' `rest'
	}
	else if "`sub'" == substr("alternatives",1,max(3,`lsub')) {
		if "`rest'" != "" {
			di as error "invalid syntax"
			exit 198
		}
		Alternatives
	}
	else if "`sub'" == "mfx" {
		Mfx `rest'
	}
	else estat_default `0'
end

program Covariance, rclass
	syntax namelist(name=which id="matrix type" max=1), ///
		[FORmat(string) BORder(string) left(numlist max=1 >=0)]

	if `e(cholesky)' Cov1 1
	else Cov2
	
	tempname V L 
	matrix `V' = r(V)

	if "`which'" == "cov" {
		matrix `V' = (`V'+`V'')*.5
		matrix `L' = `V'
		return matrix cov = `V'
		if ("`format'" == "") local format %9.0g
	}
	else if "`which'" == "cor" {
		tempname D
		matrix `D' = syminv(cholesky(diag(vecdiag(`V'))))
		matrix `L' = `D'*`V'*`D'

		matrix rownames `L' = `names'
		matrix colnames `L' = `names'
		matrix `L' = (`L' + `L'')*.5
		matrix `V' = `L'

		return matrix cor = `V'

		if ("`format'" == "") local format %6.3f

	}
	if ("`border'" == "") local border all
	if ("`left'" == "") local left 2
	matlist `L', format(`format') border(`border') left(`left') 
	if !e(structcov) {
		local jj = e(i_base)
		local ab : word `jj' of `e(alteqs)'
		if ("`which'" == "cov")	local cov covariances
		else local cov correlations
		di `"Note: `cov' are for alternatives differenced with `ab'"'
	} 
end

program Cov1, rclass
	args pivot b

	tempname L corpat stdpat corfix stdfix
	matrix `corpat' = e(corpattern)
	matrix `stdpat' = e(stdpattern)
	matrix `corfix' = e(corfixed)
	matrix `stdfix' = e(stdfixed)
	if "`b'" == "" {
		tempname b
		matrix `b' = e(b)
	}
	local dim = colsof(e(stdfixed))
	matrix `L' = I(`dim')

	forvalues j=1/`dim' {
		local k = el(e(stdpattern),1,`j')
		if `k' < . {
			local k = colnumb(`b',"lnl`j'_`j':") 
			matrix `L'[`j',`j'] = exp(`b'[1,`k'])
		}
		else matrix `L'[`j',`j'] = el(e(stdfixed),1,`j')

		local j1 = `j'+1
		forvalues i=`j1'/`dim' {
			local k = el(e(corpattern),`i',`j')
			if `k' < . {
				local k = colnumb(`b',"l`i'_`j':")
				matrix `L'[`i',`j'] = `b'[1,`k']
			}
			else matrix `L'[`i',`j'] = el(e(corfixed),`i',`j')
		}
	}
	tempname V
	matrix `L' = `L'*`L''
	matrix `V' = J(`dim',`dim',0)
	local order : rownames e(corpattern)
	forvalues j=1/`dim' {
		/* cholesky pivots base and scale alternatives 	*/
		/* rownames identify the order 			*/
		if `pivot' {
			local jj : word `j' of `order'
			if !e(structcov) & `jj'>e(i_base) {
				local `--jj' 
			}
		}
		else  local jj = `j'

		matrix `V'[`jj',`jj'] = `L'[`j',`j']


		local j1 = `j' + 1
		forvalues i=`j1'/`dim' {
			if `pivot' {
				local ii : word `i' of `order'
				if !e(structcov) & `ii'>e(i_base) {
					local `--ii' 
				}
			}
			else local ii = `i'

			matrix `V'[`ii',`jj'] = `L'[`i',`j']
			matrix `V'[`jj',`ii'] = `V'[`ii',`jj']
		}
	}
	if `pivot' {
		if e(structcov) {
			matrix rownames `V' = `e(alteqs)'
			matrix colnames `V' = `e(alteqs)'
		}
		else {
			forvalues i=1/`=e(k_alt)' {
				if (`i'==e(i_base)) continue
				local ai : word `i' of `e(alteqs)'
				local names `"`names' `ai'"'
			}
			matrix rownames `V' = `names'
			matrix colnames `V' = `names'
		}
	}
	else {
		forvalues i=1/`dim' {
			local j : word `i' of `order'
			local aj : word `j' of `e(alteqs)'
			local names `"`names' `aj'"'
		}
		matrix rownames `V' = `names'
		matrix colnames `V' = `names'
	}
	
	return matrix V = `V'
end

program Cov2, rclass
	args b

	tempname rho sigma
	local nrho = e(k_rho)
	if (`nrho' > 0)	{
		matrix `rho' = J(1,`nrho',.)
	}	
	else {
		matrix `rho' = J(1,1,.)
	}
	local nsigma = e(k_sigma)
	if (`nsigma' > 0) {
		matrix `sigma' = J(1,`nsigma',.)
	}
	else {
		matrix `sigma' = J(1,1,.)
	}

	tempname L D
	if "`b'" == "" {
		tempname b
		matrix `b' = e(b)
	}
	local nalt = e(k_alt)
	matrix `L' = I(`nalt')
	matrix `D' = I(`nalt')

	forvalues j=1/`nalt' {
		local k = el(e(stdpattern),1,`j')
		if `k' < . {
			if `e(fullcov)' {
				local k = colnumb(`b',"lnsigma`j':") 
				matrix `D'[`j',`j'] = exp(`b'[1,`k'])
			}
			else {
				if `sigma'[1,`k'] >= . {
					local i = colnumb(`b',"lnsigmaP`k':")
					matrix `sigma'[1,`k'] = exp(`b'[1,`i'])
				}
				matrix `D'[`j',`j'] = `sigma'[1,`k']
			}
		}
		else {
			matrix `D'[`j',`j'] = el(e(stdfixed),1,`j')
		}
		local j1 = `j'+1
		forvalues i=`j1'/`nalt' {
			local k = el(e(corpattern),`i',`j')
			if `k' < . {
				if `e(fullcov)' {
					local k =  colnumb(`b',"atanhr`i'_`j':") 

					matrix `L'[`i',`j'] = tanh(`b'[1,`k'])
					matrix `L'[`j',`i'] = `L'[`i',`j']
				}
				else {
					if `rho'[1,`k'] >= . {
						local k1 = colnumb(`b',"atanhrP`k':")
						matrix `rho'[1,`k'] = tanh(`b'[1,`k1'])
					}
					matrix `L'[`i',`j'] = `rho'[1,`k']
					matrix `L'[`j',`i'] = `rho'[1,`k']
				}
			}
			else {
				matrix `L'[`i',`j'] = el(e(corfixed),`i',`j')
				matrix `L'[`j',`i'] = `L'[`i',`j']
			}
		}
	}

	matrix `L' = `D'*`L'*`D'
	matrix rownames `L' = `e(alteqs)'
	matrix colnames `L' = `e(alteqs)'

	return matrix V = `L'
end

program Alternatives, rclass

	tempname table
	.`table'  = ._tab.new, col(6)
	.`table'.width |10 11 18|11 11 11|
	.`table'.strcolor . . yellow . . .
	.`table'.numcolor . yellow . yellow yellow yellow
	.`table'.numfmt . %9.0g . %10.0g %10.0g %10.2f
	.`table'.strfmt %6s . . . . .
	di _n as text "{col 4}Alternatives summary"
	.`table'.sep, top
	.`table'.titles "Covariance" "Alt " "Alt " "Cases " "Frequency" "Percent"
	.`table'.titles "index  " "value" "label" "present" "selected" "selected"
	.`table'.sep, mid

	local nalt = rowsof(e(stats))
	local k = 0
	forvalues i=1/`nalt' {
		if !`e(structcov)' & `i'==`e(i_base)' {
			local index (base)
			.`table'.strcolor green . . . . .
		}
		else {
			local index `++k'
			.`table'.strcolor yellow . . . . .
		}
		local label = abbrev("`e(alt`i')'",17)
		.`table'.row "`index'" el(e(stats),`i',1) "`label'" 	///
		 	el(e(stats),`i',2) el(e(stats),`i',3)  		///
			100*el(e(stats),`i',4)
	}
	.`table'.sep,bot
end
	
program Mfx, rclass
	syntax [if] [in], [ Level(cilevel) noDISCrete VARlist(string) 	///
		AT(string)  noWght noEsample INTBurn(integer -1) 	///
		INTPoints(integer 0) INTSeed(string) ANTIthetics FORCE ]

	/* undocumented options: intburn intpoint intseed antithetics force */

	preserve 
	tempname altern alabel touse

	marksample touse
	markout `touse' `e(altvar)' `e(indvars)' `e(ind2vars)' `e(casevar)', strok
	if "`esample'" == "" {
		qui count if e(sample)
		if (r(N) > 0) qui replace `touse' = (`touse' & e(sample))
	}
	qui keep if `touse'
	if (_N == 0) error 2000

	if `"`varlist'"' != "" {
		GetVarlists `"`varlist'"'
		local indvars `r(indvars)'
		local ind2vars `r(ind2vars)'
		/* index into original varlist */
		local i_indvars `r(i_indvars)'
		local i_ind2vars `r(i_ind2vars)'
	}
	else {
		local indvars `e(indvars)'
		local ind2vars `e(ind2vars)'
		numlist "1/`=e(k_indvars)'"
		local i_indvars `r(numlist)'
		numlist "1/`=e(k_ind2vars)'"
		local i_ind2vars `r(numlist)'
	}
	local k_indvars : word count `indvars'
	local k_ind2vars : word count `ind2vars'
	if `k_indvars' + `k_ind2vars' == 0 {
		di as error "no variables"
		exit 498
	}

	if "`at'" != "" {
		ParseAtStat "`at'"
		local atstat `r(atstat)'
		local at `"`r(at)'"'
	}
	if ("`atstat'"=="") local atstat mean

	EgenGroupAltvar `altern' `alabel' `touse' `force'
	qui keep if `touse'
	if (_N == 0) error 2000

	if "`discrete'" == "" {
		DiscreteVarIndex `"`indvars'"'
		local iIa `r(ind)'
		DiscreteVarIndex `"`ind2vars'"'
		local iIc `r(ind)'
	}
	if ("`wght'"=="" & "`e(wtype)'"!="") local wtopt [`e(wtype)'`e(wexp)']

	collapse (`atstat') `e(indvars)' `e(ind2vars)' (count) `touse' ///
		`wtopt', by(`altern') fast
	if _N == 1 {
		di as error "{p}there is only a single alternative; marginal " ///
		 "effects cannot be computed{p_end}"
		exit 459
	}

	/* k_alt is the number of alternatives for this case <= e(k_alt) */
	local k_alt = _N

	if "`at'"!="" & "`at'"!="mean" & "`at'"!="median" {
		ParseEvalAt `"`at'"' "`altern'" 
	}

	if ("`intseed'"=="") local intseed = "`e(mc_seed)'" 
	if (`intburn' > 0 ) local burn = `intburn'
	else local burn = e(mc_burn)

	if (`intpoints'>0) local npts = `intpoints'
	else local npts = e(mc_points)

	if ("`antithetics'"!="") local anti = 1
	else local anti = e(mc_antithetics)

	if (substr("`e(mc_method)'",1,3)=="Hal") local method = 1
	else if (substr("`e(mc_method)'",1,3)=="Ham") local method = 2
	else local method = 3

	if ("`level'" == "") local level `c(level)'

	tempname R V g gv pr b ai aj dx dxi  
	tempname vdp v table subt atable ctable x J 
	tempvar eta

	if (e(cholesky)) Cov1 0 
	else Cov2

	matrix `R' = r(V)
	local order : colnames e(corpattern)
	if !e(structcov) {
		local order `=e(i_base)' `order'
	}

	matrix `b' = e(b)
	matrix `V' = e(V)
	Eta `b' `eta' `altern' 
	if e(k_indvars) > 0 {
		tempname bi
		matrix `bi' = `b'[1,"`e(altvar)':"]
	}
	if e(k_ind2vars)+e(const) > 0 {
		tempname A
		forvalues j=1/`=e(k_alt)' {
			if (`j' == e(i_base)) continue
			local eqa : word `j' of `e(alteqs)'
			GetCoef "`eqa'"  
			matrix `A' =  (nullmat(`A'),r(b)')
		}
	}
	mata: _mprobit_mfx("`eta'","`altern'","`R'","`order'",1.0e-6,`method', ///
		`npts',`burn',`anti',"`bi'","`A'")
	if $MPROBIT_NUMDERIVERROR {
		di as text "note: one or more numeric derivatives failed; standard " ///
		 "errors may be questionable"
	}
	matrix `g' = r(g)
	matrix `pr' = r(p)
	matrix `gv' = r(v)
	forvalues i=1/`k_alt' {
		tempname dpdx`i'
		matrix `dpdx`i'' = r(dpdx`i')
	}
	forvalues i=1/`k_alt' {
		if `k_indvars' > 0 {
			tempname dgdb`i'
			matrix `dgdb`i'' = r(dgdb`i')
		}
		if `k_ind2vars' > 0 {
			tempname dgda`i' 
			matrix `dgda`i'' = r(dgda`i')
		}
	}
	local ca = _N
	if `"`iIa'"' != "" {
		DiscreteAltv_mfx `"`indvars'"' `"`iIa'"' `b' `V' `R' `altern' `"`order'"' ///
			`method' `npts' `burn' `anti'  

		forvalues i=1/`k_alt' {
			tempname gIa`i' vgIa`i'
			matrix `vgIa`i'' = r(vg`i')
			matrix `gIa`i'' = r(g`i')
		}
	}
	if `"`iIc'"' != "" {
		DiscreteCasev_mfx `"`ind2vars'"' `"`iIc'"' `b' `V' `R' `altern' `"`order'"' ///
			`method' `npts' `burn' `anti'  

		tempname gIc vgIc
		matrix `gIc' = r(g)
		matrix `vgIc' = r(vg)	
	}
	local nb = colsof(`b') 
	local _cil `=subinstr("`level'",".","_",1)'
	di in gr _n "Marginal effects after asmprobit" 

	forvalues i=1/`k_alt' {
		local ia = `altern'[`i']
		local rownames
		cap mat drop `table'
		if `k_indvars' {
			cap mat drop `atable'
			if "`iIa'" != "" {
				local kI = 1
				local jI : word 1 of `iIa'
				local nI : word count `iIa'
			}
			else {
				local jI = -1
			}
			matrix `vdp' = vecdiag(`dgdb`i'''*`V'*`dgdb`i'')'
			local rspec &

			forvalues j=1/`k_indvars' {
				cap mat drop `v'
				cap mat drop `dx'
				local varj : word `j' of `indvars'
				mkmat `varj', matrix(`x')
				if `j' == `jI' {
					forvalues k=1/`k_alt' {
						local ka = `altern'[`k']
						local eqa : word `ka' of `e(alteqs)'
						local rownames `rownames' `varj'*:`eqa'
						if (`k' < `k_alt') local rspec `rspec'&
					}
					matrix `v' = `vgIa`i''[1..`k_alt',`j']
					matrix `dx' = `gIa`i''[1..`k_alt',`j']
					
					if `++kI' <= `nI' {
						local jI : word `kI' of `iIa'
					}
				}
				else {
					local ji : word `j' of `i_indvars'
					local ji = (`ji'-1)*e(k_alt)+1
					forvalues k=1/`k_alt' {
						local ka = `altern'[`k']
						local eqa : word `ka' of `e(alteqs)'
						local rownames `rownames' `varj':`eqa'
						matrix `dx' = (nullmat(`dx') \    ///
							`dpdx`i''[`=`ji'+`ka'-1',1])
						matrix `v' = (nullmat(`v') \      ///
							`vdp'[`=`ji'+`ka'-1',1])
						if (`k' < `k_alt') local rspec `rspec'&
					}
				}
				if (`j'<`k_indvars' | `k_ind2vars') local rspec `rspec'|
				else local rspec `rspec'&

				mata:_mprobit_mfx_stats("`dx'","`v'",`level')
				matrix `subt' = (`dx', r(se), r(z), r(p), r(lb), r(ub), `x')
				matrix `atable' = (nullmat(`atable') \ `subt')
			}
		}
		if `k_ind2vars' {
			cap mat drop `ctable'
			matrix `dx' = J(`k_ind2vars',1,0)
			matrix `v' = J(`k_ind2vars',1,.)
			matrix `vdp' = vecdiag(`dgda`i'''*`V'*`dgda`i'')'

			if "`iIc'" != "" {
				local kI = 1
				local jI : word 1 of `iIc'
				local nI : word count `iIc'
			}
			else {
				local jI = -1
			}
			if (!`k_indvars') local rspec &
			local j = 0
			local j1 = e(k_indvars)*e(k_alt)
			foreach k of numlist `i_ind2vars' {
				local ci : word `k' of `e(ind2vars)'
				if `++j' == `jI' {
					local rownames `rownames' casevars:`ci'*
					matrix `dx'[`j',1] = `gIc'[`i',`j']
					matrix `v'[`j',1] = `vgIc'[`i',`j']
					if `++kI' <= `nI' {
						local jI : word `kI' of `iIc'
					}
				}
				else {
					local rownames `rownames' casevars:`ci'
					matrix `dx'[`j',1] = `dpdx`i''[`=`j1'+`k'',1]
					matrix `v'[`j',1] = `vdp'[`k',1]
				}
				local rspec `rspec'&
			}
			mkmat `ind2vars', matrix(`x')
			mat `x' = `x'[1,1..`k_ind2vars']'
			mata:_mprobit_mfx_stats("`dx'","`v'",`level')
			matrix `ctable' = (`dx', r(se), r(z), r(p), r(lb), r(ub), `x')
		}
		local eqa : word `ia' of `e(alteqs)'

		di in gr _col(7) _n `"Pr(choice = `eqa') ="' _col(10)  ///
			in ye %10.0g `pr'[`i',1] 
		di in smcl in gr "{hline 13}{c TT}{hline 65}"
		di in smcl in gr /*
			*/ "variable     {c |}" _col(18) "dp/dx" /*
			*/ _col(26) /*
			*/ `"Std. Err."' _col(39) `"z"' _col(45) `"P>|z|"' /*
			*/ _col(52) `"[    `=strsubdp("`level'")'% C.I.    ]"'  /*
			*/ _col(77) `"X"' _n /*
			*/ "{hline 13}{c +}{hline 65}" 

		local fmt & %12s | %8.0g & %8.0g & %6.2f & %6.3f & %8.0g & %8.0g & %8.5g &

		matrix `table' = (nullmat(`atable') \ nullmat(`ctable'))
 		matrix colnames `table' =  _:
		matrix colnames `table' = dp/dx SE z P>|z| Lb`_cil' Ub`_cil' X
		matrix rownames `table' = `rownames'
		matlist `table', cspec(`fmt') rspec(`rspec') border(bottom) noblank ///
			nodotz names(rows) linesize(79)   
		di in smcl in gr "{hline 13}{c BT}{hline 65}"
		return matrix `eqa' = `table'
		return scalar pr_`eqa' = `pr'[`i',1]
		if "`iIa'"!="" | "`iIc'"!="" {
			di "(*) dp/dx is for discrete change of indicator variable " ///
			 "from 0 to 1"
		}
	}
end

program Eta 
	args b eta altern

	if e(k_indvars) > 0 {
		matrix score double `eta' = `b', eq(#1)		
		local eq = 1
	}
	else {
		gen double `eta' = 0		
		local eq = 0
	}
	if e(k_ind2vars)+e(const) > 0 {
		tempvar eta1
		forvalues i=1/`=e(k_alt)' {
			if (`i'==e(i_base)) continue

			matrix score double `eta1' = `b' if `altern'==`i', eq(#`++eq')
			qui replace `eta' = `eta' + `eta1' if `altern'==`i'
			qui drop `eta1'
		}
	}
end

program DiscreteAltv_mfx, rclass
	args varlist ind b V R altern order method npts burn anti

	if (`"`varlist'"'=="" | `"`ind'"'=="") exit

	local na = _N 
	local nv : word count `varlist'
	forvalues j=1/`na' {
		tempname dg0db`j' vg`j' g`j'
		matrix `vg`j'' = J(`na',`nv',.)
		matrix `g`j'' = J(`na',`nv',.)
	}	

	tempname xk p0 tmp del
	tempvar eta
	foreach j of numlist `ind' {
		local var : word `j' of `varlist' 
		forvalues k = 1/`na' {
			scalar `xk' = `var'[`k']
			qui replace `var' = 0 if _n == `k'
			Eta `b' `eta' `altern' 
			mata: _mprobit_mfx("`eta'","`altern'","`R'","`order'",0,`method', ///
				`npts',`burn',`anti')

			matrix `p0' = r(p)
			forvalues i=1/`na' {
				matrix `dg0db`i'' = r(dgdb`i')
			}
			qui replace `var' = 1 if _n == `k' 
			qui drop `eta'

			Eta `b' `eta' `altern' 
			mata: _mprobit_mfx("`eta'","`altern'","`R'","`order'", 0,`method', ///
				`npts',`burn',`anti')

			qui replace `var' = `xk' if _n == `k' 
			qui drop `eta'
			matrix `del' = r(p)-`p0'
			forvalues i=1/`na' {
				matrix `g`i''[`k',`j'] = `del'[`i',1]
				matrix `tmp' = r(dgdb`i')-`dg0db`i''
				matrix `tmp' = `tmp''*`V'*`tmp'
				matrix `vg`i''[`k',`j'] = `tmp'[1,1]
			}
		}
	}
	forvalues i=1/`na' {
		return matrix vg`i' = `vg`i''
		return matrix g`i' = `g`i''	
	}
	return local ind `ind'
end

program DiscreteCasev_mfx, rclass
	args varlist ind b V R altern order method npts burn anti

	if (`"`varlist'"'=="" | `"`ind'"'=="") exit

	tempvar eta
	tempname g vg xi p0 tmp del
	local na = _N 
	local nv : word count `varlist'
	matrix `g' = J(`na',`nv',.) 
	matrix `vg' = J(`na',`nv',.)
	forvalues j=1/`na' {
		tempname dg0db`j' vdg`j'
		matrix `dg0db`j'' = r(dgdb`j')
	}
	local np = colsof(`b')
	foreach j of numlist `ind' {
		local var : word `j' of `varlist' 
		scalar `xi' = `var'[1]

		qui replace `var' = 0 
		Eta `b' `eta' `altern' 
		mata: _mprobit_mfx("`eta'","`altern'","`R'","`order'",0,`method', ///
			`npts',`burn',`anti')

		matrix `p0' = r(p)
		forvalues i=1/`na' {
			matrix `dg0db`i'' = r(dgdb`i')
		}
		qui replace `var' = 1 
		qui drop `eta'

		Eta `b' `eta' `altern' 
		mata: _mprobit_mfx("`eta'","`altern'","`R'","`order'",0,`method', ///
			`npts',`burn',`anti')

		qui replace `var' = `xi' 
		qui drop `eta'
		matrix `del' = r(p)-`p0'
		forvalues i=1/`na' {
			matrix `g'[`i',`j'] = `del'[`i',1]
			matrix `tmp' = r(dgdb`i')-`dg0db`i''
			matrix `tmp' = `tmp''*`V'*`tmp'
			matrix `vg'[`i',`j'] = `tmp'[1,1]
		}
	}
	return matrix vg = `vg'
	return matrix g = `g'	
	return local ind `ind'
end

program GetVarlists, rclass
	args varlist

	unab vlist : `varlist' 
	local allvars `e(indvars)' `e(ind2vars)' 
	/* make sure no invalid vars specified */
	foreach vari of varlist `vlist' {
		local found = 0
		foreach vark of varlist `allvars' {
			if "`vari'" == "`vark'" {
				local found = 1
				continue, break
			}
		}
		if !`found' {
			di as error "{p}variable `vari' is not an independent variable " ///
			 "in the model{p_end}"
			exit 322
		}
	}
	/* do a little extra work to keep variables in order of the */
	/* specified model 					    */
	if e(k_indvars) {
		local i = 0
		foreach vari of varlist `e(indvars)' {
			local `++i'
			local k = 0
			foreach vark of varlist `vlist' {
				local `++k'
				if "`vark'" == "`vari'" {
					local indvars `indvars' `vari'
					local i_indvars `i_indvars' `i'
					continue, break
				}
			}
		}
	}

	if e(k_ind2vars) {
		local i = 0
		foreach vari of varlist `e(ind2vars)' {
			local `++i'
			local k = 0
			foreach vark of varlist `vlist' {
				local `++k'
				if "`vark'" == "`vari'" {
					local ind2vars `ind2vars' `vari'
					local i_ind2vars `i_ind2vars' `i'
					continue, break
				}
			}
		}
	}

	return local indvars `"`indvars'"'
	return local i_indvars `"`i_indvars'"'
	return local ind2vars `"`ind2vars'"'
	return local i_ind2vars `"`i_ind2vars'"'
end

program GetCoef, rclass
	args eq

	tempname b b1
	
	matrix `b' = e(b)
	if e(k_ind2vars) {
		foreach var of varlist `e(ind2vars)' {
			matrix `b1' = (nullmat(`b1'),`b'[1,"`eq':`var'"])
		}
	}
	if e(const) == 1 {
		matrix `b1' = (nullmat(`b1'),`b'[1,"`eq':_cons"])
	}
	return matrix b = `b1' 
end

program ParseAtStat, rclass
	args at

	gettoken first at1: at, parse(" :=")
	if `"`at1'"' != "" {
		/* make sure it is not a variable specification 	*/
		cap ParseAt `"`at1'"'
		if _rc {
			return local at `"`at'"'
			exit
		}
	}
	if "`first'"!="mean" & "`first'"!="median" {
		di as error "{p} invalid statistic specified in at(), only mean " ///
		 "or median is allowed; see help "				  ///
	 	 `"{helpb asmprobit postestimation##estatmfx:asmprobit postestimation}{p_end}"'
		exit 198
	}
	return local atstat `first'
	return local at `"`at1'"'
end

program ParseEvalAt
	args at altvar
	
	local na = _N
	tempname val

	while `"`at'"' != "" {
		ParseAt `"`at'"'
		local var `r(var)'
		local alt `r(alt)'
		scalar `val' = r(val)
		local at `r(at)'
		local if
		local found = 0
		if e(k_indvars) > 0  {
			foreach vari of varlist `e(indvars)' {
				if "`var'" == "`vari'" {
					local found = 1
					continue, break
				}
			}
		}
		if !`found' {
			if e(k_ind2vars) > 0 {
				foreach vari of varlist `e(ind2vars)' {
					if "`var'" == "`vari'" {
						local found = 1
						continue, break
					}
				}
			}
			if `found' & "`alt'"!="" {
				di as error "{p}specified alternative `alt' for case " ///
				 "specific variable `var'{p_end}"
				exit 322
			}
		}
		if !`found'  {
			di as error "variable `var' is not in the model"
			exit 322
		}
		if "`alt'" != "" {
			/* alternative specific variable 		*/
			local ia
			forvalues i=1/`=e(k_alt)' {
				if "`alt'" == "`e(alt`i')'" {
					local ia = `i'
					continue, break	
				}
			}
			if "`ia'" == "" {
				forvalues i=1/`=e(k_alt)' {
					local alti : word `i' of `e(alteqs)'
					if "`alt'" == "`alti'" {
						local ia = `i'
						continue, break	
					}
				}
			}
			if "`ia'" == "" {
				di as error "`alt' is not one of the alternatives"
				exit 322
			}
			forvalues i=1/`na' {
				if `ia' == `altvar'[`i'] {
					local ia = `i'
					continue, break
				}
			}
			local if if _n == `ia'
		}
		qui replace `var' = `val' `if' 
	}	
end

program ParseAt, rclass
	args at

	local 0 `"`at'"'
	local tokens1 ": =" 
	local tokens2 ": =" 
	local tokens3 " ="
	local tokens4 " "
	tempname val
	local st = 1
	local st1 = 2
	while `"`0'"' != "" {
		gettoken tok0 0: 0, parse("`tokens`st''")
		if `st' == 1 {
			/* variable name or alternative */
			if ("`tok0'"=="") AtParseError "`at'"
			local tok1 "`tok0'"
			local st = `st1'
		}
		else if `st' <= 3 {
			if "`tok0'" == "=" {
				/* `tok1' is a variable name */
				local st = 4
				unab var : `tok1'	
				return local var "`var'"
			}
			else if `st'==2 & "`tok0'"==":" {
				/* `tok1' is an alternative */
				local st = 1
				local st1 = 3
				return local alt "`tok1'"
			}
			else AtParseError "`at'"
		}
		else if `st' == 4 {
			/* rhs */
			cap scalar `val' = `tok0'
			if (_rc) AtParseError "`at'" 
			return scalar val = `val'
			return local at `"`0'"'
			exit
		}
		else AtParseError "`at'"
	}
	AtParseError "`at'"
end

program AtParseError
	args exp

	di as error `"{p}invalid at() expression "`exp'"; see help "'  ///
	 `"{helpb asmprobit postestimation##estatmfx:asmprobit postestimation}{p_end}"'

	exit 198
end
 
program EgenGroupAltvar, rclass
	args altern alabel touse force

	if "`force'" != "" {
		local altern1 `altern'
		local alabel1 `alabel'
	}
	else {
		tempvar altern1
		tempname alabel1
	}

	cap egen long `altern1' = group(`e(altvar)'), lname(`alabel1') 
	if _rc > 0 {
		/* should not happen */
		di as error "failed to renumber alternative levels"
		exit _rc
	}
	qui label list `alabel1'
	local k_alt = r(k)
	sort `e(casevar)' `altern1'
	if "`force'" != "" {
		/* undocumented option */
		if `k_alt' > e(k_alt) {
			di in gr "{p 0 7}note: some cases have more than `=e(k_alt)' " ///
			 "alternatives; these cases will be excluded from the "        ///
			 "computations{p_end}"

			tempvar ni ni1 
			qui by `e(casevar)' : gen `ni' = sum(`touse') 
			forvalues i=`=`e(k_alt)'+1'/`k_alt' {
				qui replace `touse' = 0 if (`altern'==`i')
			}
			qui by `e(casevar)' : gen `ni1' = sum(`touse')
			qui by `e(casevar)': replace `touse' = 0 if `ni1'[_N]!=`ni'[_N]
		}
		exit
	}
	local i = 1
	local n = 0
	local k = e(k_alt)
	qui gen long `altern' = .
	local bmes = 1
	tempvar ni ni1 
	qui by `e(casevar)' : gen `ni' = sum(`touse') 
	forvalues i=1/`k_alt' {
		local alti : label `alabel1' `i'
		local found = 0
		forvalues j=1/`=e(k_alt)' {
			if `"`e(alt`j')'"' == `"`alti'"' {
				qui replace `altern' = `j' if `altern1'==`i'
				local labdef `labdef' `j' `"`alti'"'
				local found = 1
				local `++n'
				continue, break
			}
		}
		if !`found' {
			if `bmes' {
				di in gr "{p 0 7}note: some cases have alternatives " ///
				 "that are not in the fitted model; these cases "  ///
				 "will be excluded from the computations{p_end}"
				local bmes = 0
			}
			qui replace `touse' = 0 if `altern1' == `i' 
		}
	}
	if `n' == 0 {
		di as error "{p}the alternatives used in the estimation " ///
		 "sample do not match those in variable `e(altvar)'{p_end}"
		exit 322
	}
	qui by `e(casevar)' : gen `ni1' = sum(`touse')
	qui by `e(casevar)': replace `touse' = 0 if `ni1'[_N]!=`ni'[_N]
	qui label define `alabel' `labdef'
	qui label values `altern' `alabel'
end

program DiscreteVarIndex, rclass
	args varlist

	if (`"`varlist'"' == "") exit

	local nv = 0
	foreach var of varlist `varlist' {
		local `++nv''
		cap assert `var'==0 | `var'==1 
		if _rc == 0 {
			local ind `ind' `nv'
		}
	}
	return local ind `"`ind'"'
end
	
exit
