*! version 2.0.1  25apr2006
/* predict for asmprobit */
program define asmprobit_p, sortpreserve
        version 9
	
	if "`e(cmd)'" != "asmprobit" {
		di as error "{p}multinomial probit estimates from " ///
		 "{help asmprobit##|_new:asmprobit} not found{p_end}"
		exit 301
	}
        syntax newvarname [if] [in] [, XB STDP Pr FORCE]
	local predtype "`xb' `stdp' `pr'"        
        local nopt : word count `predtype'
        if `nopt' > 1 {
                di as error "only one of pr, xb, or stdp can be specified"
                exit 198
        }
	local predtype = ltrim(rtrim("`predtype'"))
	if "`predtype'" == "" {
		di in gr "(option pr assumed; Pr(`e(altvar)'))"
		local predtype "pr"
	}

	marksample touse, novarlist
	markout `touse' `e(indvars)' `e(ind2vars)'
	if "`predtype'" == "pr" {
		markout `touse' `e(altvar)' `e(casevar)', strok
	}
	qui count if `touse'
	if r(N) == 0 {
		di as error "no cases remain after excluding invalid observations"
		exit 2000
	}

	tempvar altern 
	tempname alabel levels
	local altvar `e(altvar)'
	local alteqs `"`e(alteqs)'"'
	local nalt = e(k_alt)
	EgenGroupAltvar `altern' `alabel' `touse' `force'

	sort `e(casevar)' `altern'
	if "`predtype'" == "pr" {
		tempvar ni  
		summarize `touse', meanonly
		local nu = r(sum)
		qui by `e(casevar)' : gen `ni' = sum(`touse') 
		/* get rid of singletons */
		qui by `e(casevar)': replace `touse' = 0 if `ni'[_N]==1
		summarize `touse', meanonly
		if r(sum) < `nu' {
			di in gr "note: cases with a single alternative have " ///
			 "been dropped"
		}
	}
	else {
		summarize `touse', meanonly
	}
	if r(sum) == 0 {
		di as error "no cases remain after excluding invalid observations"
		exit 2000
	}
	local neq : word count `e(ind2vars)'
	local neq = `neq' + e(const)

	tempvar result
	if "`predtype'"=="xb" {
		tempvar vi
		qui _predict `typlist' `result' if `touse', `predtype' equation(`altvar')

		if `neq' > 0 {
			forvalues i=1/`nalt' {
				if (`i' == `e(i_base)') continue

				local eq : word `i' of `alteqs'
				qui _predict `vi' if `touse' & `altern'==`i', `predtype' ///
					equation(`eq')
				qui replace `result' = `result' + `vi' if `altern'==`i' & `touse'
				drop `vi'
			}
		}
		label var `result' `"xb for each alternative in variable `altvar'"'
	}
	else if "`predtype'"=="stdp" {
		local neq : word count `e(ind2vars)'
		local neq = `neq' + e(const)
		if `neq' == 0 {
			qui _predict `typlist' `result' if `touse', `predtype' equation(`altvar')
		}
		else {
			cap confirm variable `e(depvar)'
			local rmdep = (_rc > 0)
			if `rmdep' {
				/* predictnl requires depvar to exist in call to -tsrevar- */
				qui gen `e(depvar)' = 0
			}
			local ibase `e(i_base)'

			cap {
			qui _predict `typlist' `result' if `touse' & `altern'==`ibase', ///
				`predtype' equation(`altvar')

			local i = 0
			tempvar ti vi
			foreach eq of local alteqs {
				if (`++i' == `ibase') continue
				qui predictnl `typlist' `ti'=xb(`altvar')+xb(`eq') ///	
					if `touse' & `altern'==`i', se(`vi') 
				qui replace `result' = `vi' if `touse' & `altern'==`i'
				drop `vi' `ti'
			}
			} // end cap
			local rc = _rc
			if (`rmdep') drop `e(depvar)'
			if (`rc' > 0) error `rc'
		}
		label var `result' `"Standard error of xb for each alternative in `altvar'"'
	}
	else /*if "`predtype'" == "pr"*/ {
		macro drop MPROBIT_*
		/* just in case there is an external panel info object, get rid of it */
		cap mata : mata drop _mprobit_panel_info

		global MPROBIT_TRACE 0
		global MPROBIT_CHOLESKY = e(cholesky)
		global MPROBIT_FULLCOV = e(fullcov)
		global MPROBIT_NVAR = e(k_sigma)
		global MPROBIT_NCOR = e(k_rho)
		global MPROBIT_ALTERNATIVES `altern'
		global MPROBIT_ID `e(casevar)'
		global MPROBIT_ATTRIBS `e(indvars)'
		global MPROBIT_NATTRIB = e(k_indvars)
		global MPROBIT_NCASECOEF = `neq'
		global MPROBIT_CASEVARS `e(ind2vars)'
		global MPROBIT_CONST = e(const)
		global MPROBIT_STRUCTCOV = e(structcov)
		if e(k_alt) > 2 {
			if ("`e(mc_seed)'" != "") global MPROBIT_SEED = e(mc_seed)
			if ("`e(mc_burn)'" != "") global MPROBIT_BURN = e(mc_burn)
			else global MPROBIT_BURN = 0 
			global MPROBIT_NPOINTS = e(mc_points)
			global MPROBIT_ANTITHETICS = e(mc_antithetics)
			local meth = substr("`e(mc_method)'",1,3)
			if "`meth'" == "Hal" {
				global MPROBIT_METHOD 1
			}
			else if "`meth'" == "Ham" {
				global MPROBIT_METHOD 2
			}
			else {
				global MPROBIT_METHOD 3
			}
		}
		else {
			global MPROBIT_BURN = 0 
			global MPROBIT_NPOINTS = 0
			global MPROBIT_ANTITHETICS = 0
			global MPROBIT_METHOD = 0
		}
		global MPROBIT_BASE = e(i_base)
		global MPROBIT_NALT = `nalt'
		global MPROBIT_BALANCED = 0

		tempname corstruct corfixed order 
		matrix `corfixed'  = e(corfixed)
		matrix `corstruct' = e(corpattern)
		matrix `order' = J(`nalt',1,0)
		local indices : rownames `corstruct'
		local dim = rowsof(`corfixed')
		if !e(structcov) {
			matrix `order'[1,1] = e(i_base)
			local l = 1
		}
		else local l = 0
		forvalues i=1/`dim' {
			local j : word `i' of `indices'
			matrix `order'[`=`i'+`l'',1] = `j'
			matrix `corstruct'[`i',`i'] =  el(e(stdpattern),1,`i')
			matrix `corfixed'[`i',`i'] =  el(e(stdfixed),1,`i')
		}
		global MPROBIT_CORSTRUCT `corstruct'
		global MPROBIT_CORFIXED `corfixed'
		global MPROBIT_ORDER `order'
		global MPROBIT_TOUSE `touse'

		tempname b negH lnf g 

 		scalar `lnf' = 0.0
		matrix `b' = e(b)
		cap noi {
			qui gen `typlist' `result' = .
			asmprobit_lf -1 `b' `lnf' `g' `negH' `result'
		
			label var `result' `"Probability of each alternative in variable `altvar'"'
		}
		local rc = _rc
		cap mata : mata drop _mprobit_panel_info
		macro drop MPROBIT_*
		if (`rc' > 0) exit `rc'
	}
	rename `result' `varlist'
end

program EgenGroupAltvar, rclass
	args altern alabel touse force

	if "`force'" != "" {
		local altern1 `altern'
		local alabel1 `alabel'
	}
	else {
		tempvar altern1
		tempname alabel1
	}

	cap qui egen long `altern1' = group(`e(altvar)') if `touse', lname(`alabel1') 
	if _rc > 0 {
		/* should not happen */
		di as error "failed to renumber alternative levels"
		exit _rc
	}
	qui label list `alabel1'
	local k_alt = r(k)
	sort `e(casevar)' `altern1'
	if "`force'" != "" {
		if `k_alt' > e(k_alt) {
			di in gr "{p 0 7}note: some cases have more than `=e(k_alt)' " ///
			 "alternatives; predictions set to missing for those cases{p_end}"

			tempvar ni ni1 
			qui by `e(casevar)' : gen `ni' = sum(`touse') 
			forvalues i=`=`e(k_alt)'+1'/`k_alt' {
				qui replace `touse' = 0 if (`altern'==`i')
			}
			qui by `e(casevar)' : gen `ni1' = sum(`touse')
			qui by `e(casevar)': replace `touse' = 0 if `ni1'[_N]!=`ni'[_N]
		}
		exit
	}
	local i = 1
	local n = 0
	local k = e(k_alt)
	qui gen long `altern' = .
	local bmes = 1
	tempvar ni ni1 
	qui by `e(casevar)' : gen `ni' = sum(`touse') 
	forvalues i=1/`k_alt' {
		local alti : label `alabel1' `i'
		local found = 0
		forvalues j=1/`=e(k_alt)' {
			if `"`e(alt`j')'"' == `"`alti'"' {
				qui replace `altern' = `j' if `altern1'==`i'
				local labdef `labdef' `j' `"`alti'"'
				local found = 1
				local `++n'
				continue, break
			}
		}
		if !`found' {
			if `bmes' {
				di in gr "{p 0 7}note: some cases have alternatives " ///
				 "that are not in the fitted model; predictions "  ///
				 "set to missing for those cases{p_end}"
				local bmes = 0
			}
			qui replace `touse' = 0 if `altern1' == `i' 
		}
	}
	if `n' == 0 {
		di as error "{p}the alternatives used in the estimation " ///
		 "sample do not match those in variable `e(altvar)'; "	  ///
		 "use the force option to ignore this and use the first " ///
		 "`=e(k_alt)' alternatives of `e(altvar)'{p_end}"
		exit 322
	}
	qui by `e(casevar)' : gen `ni1' = sum(`touse')
	qui by `e(casevar)': replace `touse' = 0 if `ni1'[_N]!=`ni'[_N]

	qui label define `alabel' `labdef'
	qui label values `altern' `alabel'
end
	
exit
